!-----------------------------------------------------------------------
                        module module_INIT_READ_BIN
!-----------------------------------------------------------------------
use mpi
use module_kinds
use module_dm_parallel,only : dstrb,idstrb
use module_exchange
use module_constants
use module_solver_internal_state,only: solver_internal_state
use module_microphysics_nmm
!MONARCH
use module_bsc_constants_chem
use module_bsc_chemistry_initialize,only: chemistry_initialize,chem_ic
!MONARCH
!
!-----------------------------------------------------------------------
!
      implicit none
!
      private
!
      public :: read_binary,physics_read_gwd
!
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
      contains
!
!-----------------------------------------------------------------------
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!-----------------------------------------------------------------------
!
                        subroutine read_binary &
      (INT_STATE &
      ,my_domain_id &
      ,mpi_comm_comp &
      ,mype &
      ,its,ite,jts,jte &
      ,ims,ime,jms,jme &
      ,ids,ide,jds,jde &
      ,its_h2,ite_h2,jts_h2,jte_h2 &
      ,lm &
      ,rc)
!
!-----------------------------------------------------------------------
!
use netcdf
implicit none
!
!------------------------
!***  Argument variables
!------------------------
!
type(solver_internal_state),pointer :: int_state

integer(kind=kint),intent(in) :: &
 its,ite &
,ims,ime &
,ids,ide &
,its_h2,ite_h2 &
,jts,jte &
,jms,jme &
,jds,jde &
,jts_h2,jte_h2 &
,lm &
,mpi_comm_comp &
,my_domain_id &
,mype 

integer(kind=kint),intent(out) :: &
 rc
!
!---------------------
!***  Local variables
!---------------------
!
integer(kind=kint) :: &
 i &                         ! index in x direction
,iend &
,irtn &
,j &                         ! index in y direction
,jend &
,k &                         ! index
,kount &
,ks &                        ! tracer index
,l &                         ! index in p direction
,lb &
,length &
,n &
,nl &
,nv &
,nvars_bc_2d_h &
,nvars_bc_3d_h &
,nvars_bc_4d_h &
,nvars_bc_2d_v &
,nvars_bc_3d_v &
,ub

integer(kind=kint) :: &      ! dimensions from input file
 im &
,jm &
,lmm &
,lnsh

integer(kind=kint) :: &
 iyear_fcst           &
,imonth_fcst          &
,iday_fcst            &
,ihour_fcst

real(kind=kfpt):: &
 tend,tend_max,arg

real(kind=kfpt),dimension(int_state%NSOIL) :: &
 soil1din

real(kind=kfpt),dimension(:),allocatable :: &
 all_bc_data,psint

real(kind=kfpt),allocatable,dimension(:,:) :: &
 temp1

integer(kind=kfpt),allocatable,dimension(:,:) :: &
 itemp

real(kind=kfpt),allocatable,dimension(:,:,:) :: &
 tempsoil

logical(kind=klog) :: opened

integer(kind=kint) :: &
 ierr 

character(64):: &
 infile

integer(kind=kint):: &
 ntsd &
,ntstm_max &
,nfcst
!MONARCH
character(64) :: couplefile
character(120) :: message
integer(kind=kint) :: skip2
      INTEGER(kind=kint) :: IHRST2,M
      INTEGER(kind=kint) :: NFCST2
      INTEGER(kind=kint),DIMENSION(3) :: IDAT2
      INTEGER(kind=kint),DIMENSION(:,:),ALLOCATABLE :: ITEMPDUST
      REAL(kind=kfpt),DIMENSION(:,:),ALLOCATABLE :: TEMPDUST
      REAL(kind=kfpt),DIMENSION(:,:,:),ALLOCATABLE :: TEMPDELTA
      REAL(kind=kfpt),DIMENSION(:,:,:), ALLOCATABLE :: TEMPUSTRTR
      CHARACTER(30) :: INFILE2
      LOGICAL(kind=klog) :: RUN2
      REAL(kind=kfpt),DIMENSION(:,:,:), ALLOCATABLE :: TEMPSA
      REAL(kind=kfpt),DIMENSION(:,:,:), ALLOCATABLE :: TEMP2,TEMP3,TEMP5
      REAL(kind=kfpt),DIMENSION(:), ALLOCATABLE :: MN1,MN2,MN3
      REAL(kind=kfpt),DIMENSION(:,:,:), ALLOCATABLE :: TEMP_PSDMAX
      REAL(kind=kfpt),DIMENSION(:), ALLOCATABLE :: D0, DD, USTART0, PRADI_MAX
      REAL(kind=kfpt),DIMENSION(:), ALLOCATABLE :: PMASS
      INTEGER(kind=kint),DIMENSION(:), ALLOCATABLE :: IBIN_U, IBIN_L
      INTEGER(kind=kint) :: IDST, PSDMAX
!               
      REAL(kind=kfpt),DIMENSION(:), ALLOCATABLE :: CDM,CVM,ASM,AA,RSQCD0VM
      INTEGER(kind=kint) :: stat,input_id,varid
      CHARACTER(3) :: num_var
      CHARACTER(11) :: name_variable
!MONARCH
!
!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
!
      allocate(temp1(ids:ide,jds:jde),stat=i)
!
!-----------------------------------------------------------------------
!
      select_unit: do n=51,59
        inquire(n,opened=opened)
        if(.not.opened)then
          nfcst=n
          exit select_unit
        endif
      enddo select_unit
!
!-----------------------------------------------------------------------
!
      read_blocks: if(.not.int_state%RESTART) then                     ! cold start
!
!-----------------------------------------------------------------------
!
        write(infile,'(a,i2.2)')'input_domain_',my_domain_id
        open(unit=nfcst,file=infile,status='old',form='unformatted'     &
            ,iostat=ierr)
        if(ierr/=0)then
          write(0,*)' Unable to open ',trim(infile),' in READ_BINARY'
          rc = ierr
          return
        endif
!
!-----------------------------------------------------------------------
!
        read (nfcst) int_state%RUN, &
                     int_state%IDAT, &
                     int_state%IHRST, &
                     int_state%IHREND, &
                     NTSD
        read (nfcst) int_state%PT, &
                     int_state%PDTOP, &
                     int_state%LPT2, &
                     int_state%SGM, &
                     int_state%SG1, &
                     int_state%DSG1, &
                     int_state%SGML1, &
                     int_state%SG2, &
                     int_state%DSG2, &
                     int_state%SGML2
        read (nfcst) int_state%I_PAR_STA, &
                     int_state%J_PAR_STA
        read (nfcst) int_state%DLMD, &
                     int_state%DPHD, &
                     int_state%WBD, &
                     int_state%SBD, &
                     int_state%TLM0D, &
                     int_state%TPH0D
        read (nfcst) im,jm,lmm,lnsh
!
!-----------------------------------------------------------------------
!***  Print the time & domain information.
!-----------------------------------------------------------------------
!
        if(mype==0)then
          write(0,*) 'run, idat,ntsd: ', int_state%RUN, int_state%IDAT, NTSD
          write(0,*)' Start year =',int_state%IDAT(3)
          write(0,*)' Start month=',int_state%IDAT(2)
          write(0,*)' Start day  =',int_state%IDAT(1)
          write(0,*)' Start hour =',int_state%IHRST
          write(0,*)' Timestep   =',int_state%DT
          write(0,*)' Steps/hour =',3600./int_state%DT
          if(.not.int_state%GLOBAL)write(0,*)' Max fcst hours=',int_state%IHREND
          write(0,*) 'nmm_dyn reads of PT, PDTOP: ',int_state%PT,int_state%PDTOP
          write(0,*) 'nmm_dyn reads of I_PAR_STA, J_PAR_STA: ',int_state%I_PAR_STA,int_state%J_PAR_STA
          write(0,*) 'nmm_dyn reads of TLM0D, TPH0D: ',int_state%TLM0D,int_state%TPH0D
          write(0,*) 'nmm_dyn reads of DLMD, DPHD: ',int_state%DLMD,int_state%DPHD
          write(0,*) 'nmm_dyn reads of WBD, SBD: ',int_state%WBD,int_state%SBD
          write(0,*) 'nmm_dyn reads of IM, JM, LM, LNSH: ',im,jm,lmm,lnsh
        endif
!
!-----------------------------------------------------------------------
!
      DO L=1,LM+1
        int_state%PSG1(L)=int_state%SG1(L)*int_state%PDTOP+int_state%PT
      ENDDO
      DO L=1,LM
        int_state%PDSG1(L)=int_state%DSG1(L)*int_state%PDTOP
        int_state%PSGML1(L)=int_state%SGML1(L)*int_state%PDTOP+int_state%PT
      ENDDO
!
!-- Compute pressure dependent floor values of EPSL & EPSQ2
!
      ALLOCATE(PSINT(1:LM+1))
      DO K=1,LM+1
        PSINT(K)=int_state%SG2(k)*101325.+int_state%PSG1(K)
      ENDDO
!
      DO K=2,LM
        ARG=(PSINT(K)-PSINT(2))/(PSINT(LM)-PSINT(2))*PI
!       int_state%EPSQ2(K-1)=(1.+COS(ARG))*0.09+0.02
        int_state%EPSQ2(K-1)=0.02
        int_state%EPSL(K-1)=SQRT(int_state%EPSQ2(K-1)*0.5)
      ENDDO   
      int_state%EPSQ2(LM)=int_state%EPSQ2(LM-1)
      DEALLOCATE(PSINT)

!write(0,*) 'epsl=',epsl
!write(0,*) 'epsq2=',epsq2

!
!-----------------------------------------------------------------------
!***  Proceed with getting fields from input file.
!***  NOTE: Five records were already read at the top of this routine.
!-----------------------------------------------------------------------
!
        if(mype==0)then
          read(nfcst)temp1
        endif
        do j=jms,jme
        do i=ims,ime
          int_state%fis(i,j)=0.
        enddo
        enddo
        call dstrb(temp1,int_state%fis,1,1,1,1,1,mype,mpi_comm_comp)
        call halo_exch(int_state%fis,1,2,2)
!-----------------------------------------------------------------------
!
        if(mype==0)then
          read(nfcst)temp1
        endif
        do j=jms,jme
        do i=ims,ime
          int_state%stdh(i,j)=0.
        enddo
        enddo
        call dstrb(temp1,int_state%stdh,1,1,1,1,1,mype,mpi_comm_comp)
        call halo_exch(int_state%stdh,1,2,2)
!-----------------------------------------------------------------------
!
        if(mype==0)then
          read(nfcst)temp1
        endif
        do j=jms,jme
        do i=ims,ime
          int_state%sm(i,j)=0.
        enddo
        enddo
        call dstrb(temp1,int_state%sm,1,1,1,1,1,mype,mpi_comm_comp)
        call halo_exch(int_state%sm,1,2,2)
!-----------------------------------------------------------------------
!
        if(mype==0)then
          read(nfcst)temp1
        endif
        do j=jms,jme
        do i=ims,ime
          int_state%pd(i,j)=0.
        enddo
        enddo
        call dstrb(temp1,int_state%pd,1,1,1,1,1,mype,mpi_comm_comp)
        call halo_exch(int_state%pd,1,2,2)
!-----------------------------------------------------------------------
!
        call mpi_barrier(mpi_comm_comp,irtn)
        do l=1,lm
          if(mype==0)then
            read(nfcst)temp1
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%u(i,j,l)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%u,1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%u,lm,2,2)
!-----------------------------------------------------------------------
!
        do l=1,lm
          if(mype==0)then
            read(nfcst)temp1
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%v(i,j,l)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%v,1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%v,lm,2,2)
!-----------------------------------------------------------------------
!
        do l=1,lm
          if(mype==0)then
            read(nfcst)temp1
            write(0,*) 'L, T extremes: ', L, minval(temp1),maxval(temp1)
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%t(i,j,l)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%t,1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%t,lm,2,2)
!-----------------------------------------------------------------------
!
        do l=1,lm
          if(mype==0)then
            read(nfcst)temp1
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%q(i,j,l)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%q,1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%q,lm,2,2)
!
!-----------------------------------------------------------------------
!
        do l=1,lm
          if(mype==0)then
            read(nfcst)temp1
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%cw(i,j,l)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%cw,1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%cw,lm,2,2)
!
!-----------------------------------------------------------------------
!
        do l=1,lm
          if(mype==0)then
            read(nfcst)temp1   ! O3
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%o3(i,j,l)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%o3,1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%o3,lm,2,2)
!
!-----------------------------------------------------------------------
!
        tend_max=real(int_state%IHREND)
        ntstm_max=nint(tend_max*3600./int_state%DT)+1
        tend=real(int_state%nhours_fcst)
        int_state%NTSTM=nint(tend*3600./int_state%DT)+1
        if(.not.int_state%global)then
          if(mype==0)then
            write(0,*)' Max runtime is ',tend_max,' hours'
          endif
        endif
        if(mype==0)then
          write(0,*)' Requested runtime is ',tend,' hours'
          write(0,*)' NTSTM=',int_state%NTSTM
        endif
        if(int_state%NTSTM>ntstm_max.and..not.int_state%global)then
          if(mype==0)then
            write(0,*)' Requested fcst length exceeds maximum'
            write(0,*)' Resetting to maximum'
          endif
          int_state%NTSTM=min(int_state%NTSTM,ntstm_max)
        endif
!
        int_state%ihr=nint(ntsd*int_state%DT/3600.)
!
!-----------------------------------------------------------------------
        do l=1,lm
          int_state%pdsg1(l)=int_state%dsg1(l)*int_state%pdtop
          int_state%psgml1(l)=int_state%sgml1(l)*int_state%pdtop+int_state%pt
        enddo
!
        do l=1,lm+1
          int_state%psg1(l)=int_state%sg1(l)*int_state%pdtop+int_state%pt
        enddo
!-----------------------------------------------------------------------
        do j=jts,jte
          do i=its,ite
            int_state%pdo(i,j)=int_state%pd(i,j)
          enddo
        enddo
        call halo_exch(int_state%pdo,1,2,2)
!
        do l=1,lm
          do j=jts,jte
            do i=its,ite
              int_state%up(i,j,l)=int_state%u(i,j,l)
              int_state%vp(i,j,l)=int_state%v(i,j,l)
              int_state%tp(i,j,l)=int_state%t(i,j,l)
            enddo
          enddo
        enddo
        call halo_exch(int_state%tp,lm,int_state%up,lm,int_state%vp,lm,2,2)
!-----------------------------------------------------------------------
        do l=1,lm
          do j=jms,jme
            do i=ims,ime
              int_state%q2(i,j,l)=0.02
              int_state%o3(i,j,l)=0.
              if(i.ge.ide  /2+1- 6.and.i.le.ide  /2+1+ 6.and. &
                 j.ge.jde*3/4+1- 6.and.j.le.jde*3/4+1+ 6.) then !global
!                 j.ge.jde  /2+1- 6.and.j.le.jde  /2+1+ 6.) then !regional
                int_state%o3(i,j,l)=10.
              endif
              int_state%dwdt(i,j,l)=1.
              int_state%w(i,j,l)=0.
            enddo
          enddo
        enddo
        call halo_exch(int_state%dwdt,lm,2,2)
!
        do j=jts,jte
          do i=its,ite
            int_state%pint(i,j,1)=int_state%pt
          enddo
        enddo
!
        do l=1,lm
          do j=jts,jte
            do i=its,ite
              int_state%pint(i,j,l+1)=int_state%PINT(i,j,l)+int_state%DSG2(l)*int_state%PD(i,j)+int_state%PDSG1(l)
            enddo
          enddo
        enddo
        call halo_exch(int_state%pint,lm+1,2,2)
!
        call halo_exch(int_state%q2,lm,int_state%o3,lm,2,2)
        do l=1,lm
          do j=jms,jme
            do i=ims,ime
              int_state%tracers_prev(i,j,l,int_state%indx_q )=sqrt(max(int_state%q (i,j,l),0.))
              int_state%tracers_prev(i,j,l,int_state%indx_cw)=sqrt(max(int_state%cw(i,j,l),0.))
              int_state%tracers_prev(i,j,l,int_state%indx_q2)=sqrt(max(int_state%q2(i,j,l),0.))
            enddo
          enddo
        enddo
!MONARCH
     if(int_state%num_tracers_chem>0)then
!           
!-----------------------------------------------------------------------
!***  COUPLE TRACERS - READ TRACERS (AERO+CHEM) FROM PREVIOUS RUN
!-----------------------------------------------------------------------
!
       if(mype==0)then
         write(0,*)'COUPLE_DUST:',int_state%couple_dust
       endif
!           
!-----------------------------------------------------------------------
       if(.not.int_state%couple_dust) then
!-----------------------------------------------------------------------
!
         if(int_state%num_aero>0)then
!
           do n=int_state%indx_aero_start,int_state%indx_aero_end
             do l=1,lm
               do j=jms,jme
                 do i=ims,ime
                   int_state%tracers(i,j,l,n)=0.
                 enddo
               enddo
             enddo
             call halo_exch(int_state%tracers(ims:ime,jms:jme,1:lm,n),lm,2,2)
           enddo
         endif
!
!-----------------------------------------------------------------------
!*** INITIALIZATION OF THE GAS-PHASE
!-----------------------------------------------------------------------
!
         IF(int_state%NUM_GAS_TOTAL>0) THEN
           CALL CHEM_IC(int_state%TRACERS                                       &
                       ,int_state%INDX_GAS_START,int_state%INDX_GAS_END         &
                       ,int_state%NUM_TRACERS_TOTAL                             &
                       ,IDS,IDE,JDS,JDE,LM                                      &
                       ,IMS,IME,JMS,JME                                         &
                       ,ITS,ITE,JTS,JTE)
         ENDIF !num_gas
!           
!-----------------------------------------------------------------------
       else ! couple_dust
!-----------------------------------------------------------------------
!
         ALLOCATE(tempdust(IDS:IDE,JDS:JDE),STAT=I)
!       
!-----------------------------------------------------------------------
!           
         if(int_state%couple_netcdf)then
!
           if(int_state%RECYCLE_SOIL_VARS)then
!
             couplefile='history_INIT_MET_d01.hhh.nc'
!
             if(mype==0)then
               write(0,*) 'Open ',trim(couplefile),' in netcdf'
               stat = nf90_open(couplefile,nf90_nowrite,input_id)
               message = 'opening '//trim(couplefile)//' in netcdf'
               call check_stat(stat,message)
             endif
!
             do l=1,int_state%NSOIL
               if(mype==0)then
                 print*,'RECYCLING SOIL SH2O. NSOIL= ',l
                 stat = nf90_inq_varid(input_id,'SH2O',varid)
                 message = 'No SH2O variable in file'
                 call check_stat(stat,message)
                 stat = nf90_get_var(input_id,varid,tempdust,start=(/1,1,l/),count=(/IDE,JDE,1/))
               endif
               CALL DSTRB(tempdust,int_state%SH2O(IMS:IME,JMS:JME,l),1,1,1,1,1,mype,mpi_comm_comp)
             enddo
!
             do l=1,int_state%NSOIL
               if(mype==0)then
                 print*,'RECYCLING SOIL SMC. NSOIL= ',l
                 stat = nf90_inq_varid(input_id,'SMC',varid)
                 message = 'No SMC variable in file'
                 call check_stat(stat,message)
                 stat = nf90_get_var(input_id,varid,tempdust,start=(/1,1,l/),count=(/IDE,JDE,1/))
               endif
               CALL DSTRB(tempdust,int_state%SMC(IMS:IME,JMS:JME,l),1,1,1,1,1,mype,mpi_comm_comp)
             enddo
!
             do l=1,int_state%NSOIL
               if(mype==0)then
                 print*,'RECYCLING SOIL STC. NSOIL= ',l
                 stat = nf90_inq_varid(input_id,'STC',varid)
                 message = 'No STC variable in file'
                 call check_stat(stat,message)
                 stat = nf90_get_var(input_id,varid,tempdust,start=(/1,1,l/),count=(/IDE,JDE,1/))
               endif
               CALL DSTRB(tempdust,int_state%STC(IMS:IME,JMS:JME,l),1,1,1,1,1,mype,mpi_comm_comp)
             enddo
!
             if(mype==0) stat=nf90_close(input_id)
!
           endif
!
           if(mype==0)then
             print*,'INITIALIZE AEROSOLS FROM PREVIOUS RUN'
           endif
!
           couplefile='history_INIT_TRACERS_d01.hhh.nc'
!
           if(mype==0)then
             write(0,*) 'Open ',trim(couplefile),' in netcdf'
             stat = nf90_open(couplefile,nf90_nowrite,input_id)
             message = 'opening '//trim(couplefile)//' in netcdf'
             call check_stat(stat,message)
           endif
!
           do n=int_state%num_tracers_total-int_state%num_tracers_chem+1,int_state%num_tracers_total
             do l=1,lm
               if(mype==0)then
                 write(num_var,'(I3.3)')n
                 name_variable='TRACERS_'//num_var
                 stat = nf90_inq_varid(input_id,name_variable,varid)
                 message = 'No '//trim(name_variable)//' variable in file'
                 call check_stat(stat,message)
                 stat = nf90_get_var(input_id,varid,tempdust,start=(/1,1,l/),count=(/IDE,JDE,1/))
               endif
               do j=jms,jme
                 do i=ims,ime
                   int_state%tracers(i,j,l,n)=0.
                 enddo
               enddo
               call dstrb(tempdust,int_state%tracers(:,:,:,n),1,1,1,lm,l,mype,mpi_comm_comp)
             enddo
             call halo_exch(int_state%tracers(ims:ime,jms:jme,1:lm,n),lm,2,2)
           enddo
!
           if(mype==0) stat=nf90_close(input_id)
!
         else
!
        couplefile='history_INIT.hhh'
        if(mype==0)then
        write(0,*) 'Open history_INIT.hhh in binary'
        open(unit=31,file=couplefile,status='old',access='sequential'       &
            ,form='unformatted')
        endif
!     
!-----------------------------------------------------------------------
!
!        nsoil=4
!
        if(.not.int_state%adiabatic) then
          if(int_state%RECYCLE_SOIL_VARS) then
             skip2=171+11*lm+lm+1+10*lm  !read until sh2o, so we can recycle sh20,smc,stc
          else
             skip2=171+11*lm+lm+1+10*lm+3*4
          endif
        elseif(int_state%adiabatic) then
        endif
!
        if(mype==0)then
          print*,'INITIALIZE AEROSOLS FROM PREVIOUS RUN skip: ',skip2
          do n=1,skip2
            read(31)
          enddo
        endif
        if(int_state%RECYCLE_SOIL_VARS) then
        do l=1,int_state%NSOIL
          if(mype==0)then
            print*,'RECYCLING SOIL SH2O. NSOIL= ',l
            read(31) tempdust
          endif
          CALL DSTRB(tempdust,int_state%SH2O(IMS:IME,JMS:JME,l),1,1,1,1,1,mype,mpi_comm_comp)
        enddo
!
        do l=1,int_state%NSOIL
          if(mype==0)then
            print*,'RECYCLING SOIL SMC. NSOIL= ',l
            read(31) tempdust
          endif
          CALL DSTRB(tempdust,int_state%SMC(IMS:IME,JMS:JME,l),1,1,1,1,1,mype,mpi_comm_comp)
        enddo
!
        do l=1,int_state%NSOIL
          if(mype==0)then
            print*,'RECYCLING SOIL STC. NSOIL= ',l
            read(31) tempdust
          endif
          CALL DSTRB(tempdust,int_state%STC(IMS:IME,JMS:JME,l),1,1,1,1,1,mype,mpi_comm_comp)
        enddo
        endif
!
        do l=1,int_state%num_aero
          if(mype==0)then
            read(31) tempdust                                    ! AERO_ACPREC
          endif
        enddo
!
        do l=1,int_state%num_aero
          if(mype==0)then
            read(31) tempdust                                    ! AERO_CUPREC
          endif
        enddo
!
        do l=1,int_state%num_aero
          if(mype==0)then
            read(31) tempdust                                    ! AERO_DEPDRY
          endif
!          do J=JMS,JME
!          do I=IMS,IME
!            DEPDRY(I,J,N)=0.
!          enddo
!          enddo
!          call DSTRB(TEMPDUST,DEPDRY,1,1,1,1,1)
!          call HALO_EXCH(DEPDRY,1,2,2)
        enddo

!AOD CALCULATED IN RRTM. TOTAL OF 6 AEROSOLS: DU,SS,OM,BC,SU,TOTAL NEEDS TO BE FIXED TO 6
!IF NOT BSC THIS 6 DIMENSION COMES FROM GOCART
        do l=1,6
          if(mype==0)then
            read(31) tempdust                                    ! AERO_AOD_R
          endif
        enddo

!        do l=1,lm
!          if(mype==0)then
!            read(31) tempdust                                    ! DUST_EXT
!          endif
!        enddo
!
        do l=1,int_state%num_aero
          if(mype==0)then
          read(31) tempdust                                    ! DLOAD
          endif
        enddo
!
        do l=1,int_state%num_aero
          if(mype==0)then
          read(31) tempdust                                    ! EMISS_AERO
          endif
        enddo
!
!        do l=1,int_state%num_aero
!          if(mype==0)then
!          read(31) tempdust                                    ! AERO02
!          endif
!        enddo
!
        if(int_state%num_gas_total==0)then
          if(mype==0)then        
          read(31) tempdust                                    ! ADRYDEP GAS
          read(31) tempdust                                    ! WETDEP GAS
          endif
        else
!
        do l=1,int_state%num_gas_total
          if(mype==0)then        
          read(31) tempdust                                    ! ADRYDEP GAS
          endif                  
        enddo                    
!                                
        do l=1,int_state%num_gas_total
          if(mype==0)then        
          read(31) tempdust                                    ! WETDEP GAS
          endif
        enddo
!
        endif
!
        do l=1,lm
          if(mype==0)then
          read(31) tempdust                                    ! PHOT GAS
          endif
        enddo
!
        do n=1,int_state%num_water !int_state%num_tracers_total-int_state%num_tracers_chem                !SKIP met tracers and water array
        do l=1,lm
          if(mype==0)then
            read(31) temp1
          endif
        enddo
        enddo

        do n=int_state%num_tracers_total-int_state%num_tracers_chem+1,int_state%num_tracers_total
        do l=1,lm
          if(mype==0)then
            read(31) temp1
          endif
          do j=jms,jme
          do i=ims,ime
            int_state%tracers(i,j,l,n)=0.
          enddo
          enddo
          call dstrb(temp1,int_state%tracers(:,:,:,n),1,1,1,lm,l,mype,mpi_comm_comp)
        enddo
        call halo_exch(int_state%tracers(ims:ime,jms:jme,1:lm,n),lm,2,2)
        enddo
!
!        if(mype==0)then
!          read(31)                                               ! FACT10
!        endif
!
        if(mype==0)then
          close(31)
        endif
!
        endif
!
!-----------------------------------------------------------------------
!
        DEALLOCATE(tempdust)
!
!-----------------------------------------------------------------------
         endif !couple_dust
!-----------------------------------------------------------------------
!
         do n=int_state%num_tracers_total-int_state%num_tracers_chem+1,int_state%num_tracers_total
           do l=1,lm
             do j=jms,jme
               do i=ims,ime
                 int_state%tracers_prev(i,j,l,n  )=sqrt(max(int_state%tracers(i,j,l,n),0.))
               enddo
             enddo
           enddo
         enddo
!
       endif ! endif num_tracers_chem 
!
!NOW collect or read densities and radii of each aerosol type 
!from module_CONSTANTS_CHEM.F90
     if (int_state%num_aero>0)then
!
        if(int_state%num_ash>0)then
         do n=1,int_state%num_ash
          int_state%rho_aero(n) = 0.
          int_state%r_aero_v(n) = 0.
          int_state%r_aero_e(n) = 0.
         enddo
        endif !endif num_ash
!
        m=int_state%num_ash
!
        if(int_state%num_dust>0)then
         do n=1,int_state%num_dust
          int_state%rho_aero(m+n) = rho_dust(n)
          int_state%r_aero_v(m+n) = r_dust_v(n)
          int_state%r_aero_e(m+n) = r_dust_e(n)
         enddo
        endif !endif num_dust
!
        m=m+int_state%num_dust
!
        if(int_state%num_salt>0)then
         do n=1,int_state%num_salt
          int_state%rho_aero(m+n) = rho_salt(n)
          int_state%r_aero_v(m+n) = r_salt_v(n)
          int_state%r_aero_e(m+n) = r_salt_e(n)
         enddo
        endif !endif num_salt
!
        m=m+int_state%num_salt
!
        if(int_state%num_om>0)then
         do n=1,int_state%num_om
          int_state%rho_aero(m+n) = rho_om(n)
          int_state%r_aero_v(m+n) = r_om_v(n)
          int_state%r_aero_e(m+n) = r_om_e(n)
         enddo
        endif !endif num_om
!
        m=m+int_state%num_om
!
        if(int_state%num_bc>0)then
         do n=1,int_state%num_bc
          int_state%rho_aero(m+n) = rho_bc(n)
          int_state%r_aero_v(m+n) = r_bc_v(n)
          int_state%r_aero_e(m+n) = r_bc_e(n)
         enddo
        endif !endif num_bc
!
       m=m+int_state%num_bc
!
        if(int_state%num_so4>0)then
         do n=1,int_state%num_so4
          int_state%rho_aero(m+n) = rho_so4(n)
          int_state%r_aero_v(m+n) = r_so4_v(n)
          int_state%r_aero_e(m+n) = r_so4_e(n)
         enddo
        endif  !endif num_so4
!
       m=m+int_state%num_so4
!
        if(int_state%num_no3>0)then
         do n=1,int_state%num_no3
          int_state%rho_aero(m+n) = rho_no3(n)
          int_state%r_aero_v(m+n) = r_no3_v(n)
          int_state%r_aero_e(m+n) = r_no3_e(n)
         enddo
        endif  !endif num_no3
!
       m=m+int_state%num_no3
!
        if(int_state%num_nh4>0)then
         do n=1,int_state%num_nh4
          int_state%rho_aero(m+n) = rho_nh4(n)
          int_state%r_aero_v(m+n) = r_nh4_v(n)
          int_state%r_aero_e(m+n) = r_nh4_e(n)
         enddo
        endif  !endif num_nh4
!
       m=m+int_state%num_nh4
!
        if(int_state%num_unspc>0)then
         do n=1,int_state%num_unspc
          int_state%rho_aero(m+n) = rho_unspc(n)
          int_state%r_aero_v(m+n) = r_unspc_v(n)
          int_state%r_aero_e(m+n) = r_unspc_e(n)
         enddo
        endif  !endif num_unspc
!
       m=m+int_state%num_unspc
!
        if(int_state%num_aero_uci>0)then
!         do n=1,int_state%num_aero_uci
!          rho_aero(m+n) = rho_aero_uci(n)
!          r_aero_v(m+n) = r_aero_uci_v(n)
!          r_aero_e(m+n) = r_aero_uci_e(n)
!         enddo
        endif  !endif num_aero_uci
!
!
      endif !endif num_aero
!
      if(mype==0)then
        write(0,*)'**** read in dynamics ***************'
        write(0,*)' num_aero =',int_state%num_aero
        write(0,*)' num_tracers_chem=',int_state%num_tracers_chem
        write(0,*)' num_dust =',int_state%num_dust
        write(0,*)' num_salt =',int_state%num_salt
        write(0,*)' num_om =',int_state%num_om
        write(0,*)' num_bc=',int_state%num_bc
        write(0,*)' num_so4  =',int_state%num_so4
        write(0,*)' num_no3  =',int_state%num_no3
        write(0,*)' num_nh4  =',int_state%num_nh4
        write(0,*)' num_unspc=',int_state%num_unspc
        write(0,*)' num_aero_uci =',int_state%num_aero_uci
        write(0,*)' rho_aero =',int_state%rho_aero
        write(0,*)' r_aero_v  =',int_state%r_aero_v
        write(0,*)' r_aero_e =',int_state%r_aero_e
        write(0,*)'*************************************'
      endif
!MONARCH
!
!-----------------------------------------------------------------------
!---reading surface data------------------------------------------------
!-----------------------------------------------------------------------
!
        if(mype==0)then
          read(nfcst)temp1  ! ALBEDO
        endif
      CALL DSTRB(TEMP1,int_state%ALBEDO,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! ALBASE
        endif
      CALL DSTRB(TEMP1,int_state%ALBASE,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! EPSR
        endif
      CALL DSTRB(TEMP1,int_state%EPSR,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! MXSNAL
        endif
      CALL DSTRB(TEMP1,int_state%MXSNAL,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! TSKIN
        endif
      CALL DSTRB(TEMP1,int_state%TSKIN,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! SST
        endif
      CALL DSTRB(TEMP1,int_state%SST,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! SNO and SNOWC
        endif
      CALL DSTRB(TEMP1,int_state%SNO,1,1,1,1,1,mype,mpi_comm_comp)

      DO J=JMS,JME
      DO I=IMS,IME
        if(int_state%SNO(I,J).gt.0.) then
           int_state%SNOWC(I,J)=0.98
        else
           int_state%SNOWC(I,J)=0.0
        endif
      ENDDO
      ENDDO
!
        if(mype==0)then
          read(nfcst)temp1  ! SI
        endif
      CALL DSTRB(TEMP1,int_state%SI,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst)temp1  ! SICE
        endif
      CALL DSTRB(TEMP1,int_state%SICE,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) temp1  ! TG
        endif
      CALL DSTRB(TEMP1,int_state%TG,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) temp1  ! CMC
        endif
      CALL DSTRB(TEMP1,int_state%CMC,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) temp1  ! SR
        endif
      CALL DSTRB(TEMP1,int_state%SR,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) temp1  ! USTAR
        endif
      CALL DSTRB(TEMP1,int_state%USTAR,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) temp1  ! Z0
        endif
      CALL DSTRB(TEMP1,int_state%Z0,1,1,1,1,1,mype,mpi_comm_comp)
      CALL HALO_EXCH(int_state%Z0,1,3,3)
!
        if(mype==0)then
          read(nfcst) temp1  ! Z0BASE
        endif
      CALL DSTRB(TEMP1,int_state%Z0BASE,1,1,1,1,1,mype,mpi_comm_comp)
      CALL HALO_EXCH(int_state%Z0BASE,1,3,3)
!
        if(mype==0)then
          read(nfcst) temp1  ! Z0LM
        endif
      CALL DSTRB(TEMP1,int_state%Z0LM,1,1,1,1,1,mype,mpi_comm_comp)
      CALL HALO_EXCH(int_state%Z0LM,1,3,3)
!
        if(mype==0)then
          read(nfcst) temp1  ! Z0BASELM
        endif
      CALL DSTRB(TEMP1,int_state%Z0BASELM,1,1,1,1,1,mype,mpi_comm_comp)
      CALL HALO_EXCH(int_state%Z0BASELM,1,3,3)
! If chosen, replace default z0 with z0lm 
      IF (int_state%VEGF_UNI.EQ.1 .AND. (int_state%DMASK.EQ.3 .OR. int_state%DMASK.EQ.4) ) THEN
         write(0,*) 'Replace Z0 with Z0LM'
         int_state%Z0 = int_state%Z0LM
         int_state%Z0BASE = int_state%Z0BASELM
      ENDIF
!
      ALLOCATE(TEMPSOIL(1:int_state%NSOIL,IDS:IDE,JDS:JDE),STAT=I)
!
        if(mype==0)then
          read(nfcst) TEMPSOIL  ! STC
        endif
!MONARCH
      if((.NOT.int_state%COUPLE_DUST).OR.(.NOT.int_state%RECYCLE_SOIL_VARS.AND.int_state%COUPLE_DUST)) then
      CALL DSTRB(TEMPSOIL(1,IDS:IDE,JDS:JDE),int_state%STC(IMS:IME,JMS:JME,1),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(2,IDS:IDE,JDS:JDE),int_state%STC(IMS:IME,JMS:JME,2),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(3,IDS:IDE,JDS:JDE),int_state%STC(IMS:IME,JMS:JME,3),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(4,IDS:IDE,JDS:JDE),int_state%STC(IMS:IME,JMS:JME,4),1,1,1,1,1,mype,mpi_comm_comp)
      endif
!MONARCH
!
        if(mype==0)then
          read(nfcst) TEMPSOIL  ! SMC
        endif
!MONARCH
      if((.NOT.int_state%COUPLE_DUST).OR.(.NOT.int_state%RECYCLE_SOIL_VARS.AND.int_state%COUPLE_DUST)) then
      CALL DSTRB(TEMPSOIL(1,:,:),int_state%SMC(:,:,1),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(2,:,:),int_state%SMC(:,:,2),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(3,:,:),int_state%SMC(:,:,3),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(4,:,:),int_state%SMC(:,:,4),1,1,1,1,1,mype,mpi_comm_comp)
      endif
!MONARCH
!
        if(mype==0)then
          read(nfcst) TEMPSOIL  ! SH2O
        endif
!MONARCH
      if((.NOT.int_state%COUPLE_DUST).OR.(.NOT.int_state%RECYCLE_SOIL_VARS.AND.int_state%COUPLE_DUST)) then
      CALL DSTRB(TEMPSOIL(1,:,:),int_state%SH2O(:,:,1),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(2,:,:),int_state%SH2O(:,:,2),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(3,:,:),int_state%SH2O(:,:,3),1,1,1,1,1,mype,mpi_comm_comp)
      CALL DSTRB(TEMPSOIL(4,:,:),int_state%SH2O(:,:,4),1,1,1,1,1,mype,mpi_comm_comp)
      endif
!MONARCH
!
      DEALLOCATE(TEMPSOIL)
      ALLOCATE(ITEMP(IDS:IDE,JDS:JDE),STAT=I)
!
        if(mype==0)then
          read(nfcst) ITEMP  ! ISLTYP
        endif
      CALL IDSTRB(ITEMP,int_state%ISLTYP,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) ITEMP  ! IVGTYP
        endif
      CALL IDSTRB(ITEMP,int_state%IVGTYP,mype,mpi_comm_comp)
!
      DEALLOCATE(ITEMP)
!
        if(mype==0)then
          read(nfcst) temp1  ! VEGFRC
        endif
      CALL DSTRB(TEMP1,int_state%VEGFRC,1,1,1,1,1,mype,mpi_comm_comp)
!
        if(mype==0)then
          read(nfcst) SOIL1DIN  ! DZSOIL
        endif
!
        if(mype==0)then
          read(nfcst) SOIL1DIN  ! SLDPTH
        endif
!
!       if(mype==0)then               ! here will be 14 orography fields for GWD
!         do n=1,14
!           read(nfcst) temp1
!         enddo
!       endif
!
!-----------------------------------------------------------------------
!
        close(nfcst)
!MONARCH
!read necessary inputs for mineral dust

      dust_read: IF(int_state%NUM_DUST>0) THEN
        
      select_unit3: DO N=51,59
        INQUIRE(N,OPENED=OPENED)
        IF(.NOT.OPENED)THEN
          NFCST2=N
          EXIT select_unit3
        ENDIF
      ENDDO select_unit3
      
      INFILE2='main_input_filename2_01'
!
      ALLOCATE(ITEMPDUST(IDS:IDE,JDS:JDE),STAT=I)
      ALLOCATE(TEMPDUST(IDS:IDE,JDS:JDE),STAT=I)
      ALLOCATE(TEMPSA(IDS:IDE,JDS:JDE,4),STAT=I)                             
      ALLOCATE(AA(8),STAT=I)
      ALLOCATE(CDM(int_state%NUM_VEG),STAT=I)
      ALLOCATE(CVM(int_state%NUM_VEG),STAT=I)
      ALLOCATE(RSQCD0VM(int_state%NUM_VEG),STAT=I)
      ALLOCATE(ASM(int_state%NUM_VEG),STAT=I)                                
      ALLOCATE(TEMP3(IDS:IDE,JDS:JDE,3),STAT=I)
      ALLOCATE(TEMP2(IDS:IDE,JDS:JDE,2),STAT=I)
      ALLOCATE(TEMP5(IDS:IDE,JDS:JDE,5),STAT=I)
      ALLOCATE(MN1(8),STAT=I)
      ALLOCATE(MN2(8),STAT=I)
      ALLOCATE(MN3(8),STAT=I)
      ALLOCATE(TEMP_PSDMAX(IDS:IDE,JDS:JDE,int_state%PSDMAX),STAT=I)
      ALLOCATE(DD(int_state%PSDMAX),STAT=I)
      ALLOCATE(D0(0:int_state%PSDMAX),STAT=I)
      ALLOCATE(USTART0(int_state%PSDMAX),STAT=I)
      ALLOCATE(PRADI_MAX(8),STAT=I)
      ALLOCATE(PMASS(int_state%PSDMAX),STAT=I)
      ALLOCATE(IBIN_U(8),STAT=I)
      ALLOCATE(IBIN_L(8),STAT=I)
      IF(MYPE==0)THEN
        OPEN(unit=NFCST2,file=INFILE2,status='old',form='unformatted')
        WRITE(0,*)'open NFCST2'
      ENDIF
!
      IF(MYPE==0)THEN
        READ(NFCST2)RUN2,IDAT2,IHRST2
        WRITE(0,*)'read RUN2', IDAT2, IHRST2
      ENDIF
!
      IF(MYPE==0)THEN
        READ(NFCST2) ITEMPDUST                                                ! landusenewcorr
        WRITE(0,*) 'min, max for landusenew/VEGDUST: ', minval(ITEMPDUST),maxval(ITEMPDUST)
      ENDIF
      CALL IDSTRB(ITEMPDUST,int_state%VEGDUST,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST
        write(0,*) 'min, max for BARE_SOIL_FR: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%BARE_SOIL_FR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!
!      IF(MYPE==0)THEN
!        READ(NFCST2)TEMPDUST
!      ENDIF
!      CALL DSTRB(TEMPDUST,int_state%PREF,1,1,1,1,1,MYPE,MPI_COMM_COMP)

      IF(MYPE==0)THEN
        READ(NFCST2)TEMPSA
      write(0,*) 'min, max for PREF: ', minval(TEMPSA),maxval(TEMPSA)
      ENDIF
      DO N=1,4
      CALL DSTRB(TEMPSA(:,:,N),int_state%PREF(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!     
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST
        write(0,*) 'min, max for SND: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%SND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!     
!      IF(MYPE==0)THEN
!        READ(NFCST2)TEMPDUST
!        write(0,*) 'min, max for ERODFRAC: ', minval(TEMPDUST),maxval(TEMPDUST)
!      ENDIF
!      CALL DSTRB(TEMPDUST,int_state%ERODFRAC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMP3
        write(0,*) 'min, max for ERODFRAC: ', minval(TEMP3),maxval(TEMP3)
      ENDIF
      DO N=1,3
      CALL DSTRB(TEMP3(:,:,N),int_state%ERODFRAC(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST
        write(0,*) 'min, max for BEDROCK_FR: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%BEDROCK_FR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST
      ENDIF
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST
        write(0,*) 'min, max for VEGFRC_PV: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%VEGFRC_PV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
! If chosen, replace default vegfrc with vegfrc_pv
      IF (int_state%VEGF_UNI.EQ.1 .AND. (int_state%DMASK.EQ.3 .OR. int_state%DMASK.EQ.4) ) THEN 
           int_state%VEGFRC = int_state%VEGFRC_PV
!           write(0,*) 'VEGFRC reset, min = ', minval(int_state%VEGFRC), ' max= ', maxval(int_state%VEGFRC)
      ENDIF
!     
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPSA
        write(0,*) 'min, max for SOIL_MF: ', minval(TEMPSA),maxval(TEMPSA)
      ENDIF
      DO N=1,4
      CALL DSTRB(TEMPSA(:,:,N),int_state%SOIL_MF(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPSA
        write(0,*) 'min, max for SOIL_MF_ORIG: ', minval(TEMPSA),maxval(TEMPSA)
      ENDIF
      DO N=1,4
      CALL DSTRB(TEMPSA(:,:,N),int_state%SOIL_MF_ORIG(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPSA
      write(0,*) 'min, max for SOIL_SA: ', minval(TEMPSA),maxval(TEMPSA)
      ENDIF
      DO N=1,4
      CALL DSTRB(TEMPSA(:,:,N),int_state%SOIL_SA(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPSA
      write(0,*) 'min, max for SOIL_SA_ORIG: ', minval(TEMPSA),maxval(TEMPSA)
      ENDIF
      DO N=1,4
      CALL DSTRB(TEMPSA(:,:,N),int_state%SOIL_SA_ORIG(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!      IF(MYPE==0)THEN
!        READ(NFCST2)TEMPDUST
!        write(0,*) 'min, max for ALPHA: ', minval(TEMPDUST),maxval(TEMPDUST)
!      ENDIF
!      CALL DSTRB(TEMPDUST,int_state%ALPHA,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      IF(MYPE==0)THEN
        READ(NFCST2)TEMP2
        write(0,*) 'min, max for ALPHADUST: ', minval(TEMP2),maxval(TEMP2)
      ENDIF
      DO N=1,2
      CALL DSTRB(TEMP2(:,:,N),int_state%ALPHADUST(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST
        write(0,*) 'min, max for WPRIM: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%WPRIM,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!     
      IF(MYPE==0)THEN
        READ(NFCST2)TEMP5
        write(0,*) 'min, max for USTRTR: ', minval(TEMP5),maxval(TEMP5)
      ENDIF
      DO N=1,5
        CALL DSTRB(TEMP5(:,:,N),int_state%USTRTR(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)
      ENDDO
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST                                         ! z0new
        write(0,*) 'min, max for Z0DUST: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%Z0DUST,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!     
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST                                         ! z0corr
        write(0,*) 'min, max for Z0CORR: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
!     
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST                                         ! roughness
        write(0,*) 'min, max for ROUGHNESS: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
!
      IF(MYPE==0)THEN
        READ(NFCST2)TEMPDUST                                         ! z0new
        write(0,*) 'min, max for Z0_PRIG: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%Z0_PRIG,1,1,1,1,1,MYPE,MPI_COMM_COMP)
 
      IF(MYPE==0)THEN
        READ(NFCST2) MN1
        write(0,*) 'min, max for MN1: ', minval(MN1),maxval(MN1)
        READ(NFCST2) MN2
        write(0,*) 'min, max for MN2: ', minval(MN2),maxval(MN2)
        READ(NFCST2) MN3
        write(0,*) 'min, max for MN3: ', minval(MN3),maxval(MN3)
        READ(NFCST2) AA
        write(0,*) 'min, max for AA: ', minval(AA),maxval(AA)
        READ(NFCST2) CDM
        write(0,*) 'min, max for CDM: ', minval(CDM),maxval(CDM)
        READ(NFCST2) CVM
        write(0,*) 'min, max for CVM: ', minval(CVM),maxval(CVM)
        READ(NFCST2) RSQCD0VM
        write(0,*) 'min, max for RSQCD0VM: ', minval(RSQCD0VM),maxval(RSQCD0VM)
        READ(NFCST2) ASM
        write(0,*) 'min, max for ASM: ', minval(ASM),maxval(ASM)
      ENDIF
!       
      CALL MPI_BCAST(MN1(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(MN2(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(MN3(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(AA(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(CDM(1),int_state%NUM_VEG,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(CVM(1),int_state%NUM_VEG,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(RSQCD0VM(1),int_state%NUM_VEG,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(ASM(1),int_state%NUM_VEG,MPI_REAL,0,MPI_COMM_COMP,IRTN)
! 
      IF(MYPE==0)THEN
        READ(NFCST2) TEMPDUST
        write(0,*) 'min, max for DRYSMC: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%DRYSMC,1,1,1,1,1,MYPE,MPI_COMM_COMP)      
      IF(MYPE==0)THEN
        READ(NFCST2) TEMPDUST
        write(0,*) 'min, max for A_SMOIS: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%A_SMOIS,1,1,1,1,1,MYPE,MPI_COMM_COMP)         
      IF(MYPE==0)THEN
        READ(NFCST2) TEMPDUST
        write(0,*) 'min, max for B_SMOIS: ', minval(TEMPDUST),maxval(TEMPDUST)
      ENDIF
      CALL DSTRB(TEMPDUST,int_state%B_SMOIS,1,1,1,1,1,MYPE,MPI_COMM_COMP)    
      IF(MYPE==0)THEN
         READ(NFCST2) PSDMAX
         write(0,*) 'PSDMAX: ', PSDMAX      
         if (int_state%PSDMAX .ne. PSDMAX) THEN 
            write(0,*) 'ERROR: PSDMAX differs between PREPROC and MODEL'
         endif        
      ENDIF    
      CALL MPI_BCAST(PSDMAX,1,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      int_state%PSDMAX = PSDMAX
      IF(MYPE==0)THEN
        READ(NFCST2) TEMP_PSDMAX
        write(0,*) 'min, max for PSDM: ', minval(TEMP_PSDMAX),maxval(TEMP_PSDMAX)
      ENDIF 
      DO N=1,int_state%PSDMAX     
         CALL DSTRB(TEMP_PSDMAX(:,:,N),int_state%PSDM(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)      
      ENDDO
      IF(MYPE==0)THEN
        READ(NFCST2) TEMP_PSDMAX
        write(0,*) 'min, max for PSDF: ', minval(TEMP_PSDMAX),maxval(TEMP_PSDMAX)
      ENDIF
      DO N=1,int_state%PSDMAX            
         CALL DSTRB(TEMP_PSDMAX(:,:,N),int_state%PSDF(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)    
      ENDDO
      IF(MYPE==0)THEN
        READ(NFCST2) TEMP_PSDMAX
        write(0,*) 'min, max for DPSDM: ', minval(TEMP_PSDMAX),maxval(TEMP_PSDMAX)
      ENDIF
      DO N=1,int_state%PSDMAX        
         CALL DSTRB(TEMP_PSDMAX(:,:,N),int_state%DPSDM(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)    
      ENDDO
      IF(MYPE==0)THEN
        READ(NFCST2) TEMP_PSDMAX
        write(0,*) 'min, max for DPSDF: ', minval(TEMP_PSDMAX),maxval(TEMP_PSDMAX)
      ENDIF
      DO N=1,int_state%PSDMAX  
         CALL DSTRB(TEMP_PSDMAX(:,:,N),int_state%DPSDF(:,:,N),1,1,1,1,1,MYPE,MPI_COMM_COMP)          
      ENDDO
      IF(MYPE==0)THEN      
         READ(NFCST2) IDST
         write(0,*) 'IDST: ', IDST  
      ENDIF
      IF(MYPE==0)THEN            
         READ(NFCST2) IBIN_U
         write(0,*) 'min, max for IBIN_U: ', minval(IBIN_U),maxval(IBIN_U)
      ENDIF
      IF(MYPE==0)THEN            
         READ(NFCST2) IBIN_L
         write(0,*) 'min, max for IBIN_L: ', minval(IBIN_L),maxval(IBIN_L)
      ENDIF      
      IF(MYPE==0)THEN      
         READ(NFCST2) D0
         write(0,*) 'min, max for D0: ', minval(D0),maxval(D0)
      ENDIF
      IF(MYPE==0)THEN      
         READ(NFCST2) DD
         write(0,*) 'min, max for DD: ', minval(DD),maxval(DD)
      ENDIF
      IF(MYPE==0)THEN      
         READ(NFCST2) USTART0
         write(0,*) 'min, max for USTART0: ', minval(USTART0),maxval(USTART0)
      ENDIF
      IF(MYPE==0)THEN            
         READ(NFCST2) PRADI_MAX
         write(0,*) 'min, max for PRADI_MAX: ', minval(PRADI_MAX),maxval(PRADI_MAX)
      ENDIF
      IF(MYPE==0)THEN            
         READ(NFCST2) PMASS
         write(0,*) 'min, max for PMASS: ', minval(PMASS),maxval(PMASS)    
      ENDIF
      CALL MPI_BCAST(IDST,1,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(IBIN_U(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(IBIN_L(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)      
      CALL MPI_BCAST(D0(0),int_state%PSDMAX+1,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(DD(1),int_state%PSDMAX,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(USTART0(1),int_state%PSDMAX,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(PRADI_MAX(1),8,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      CALL MPI_BCAST(PMASS,int_state%PSDMAX,MPI_REAL,0,MPI_COMM_COMP,IRTN)
      int_state%IDST      = IDST
      int_state%IBIN_U    = IBIN_U
      int_state%IBIN_L    = IBIN_L
      int_state%D0        = D0
      int_state%DD        = DD
      int_state%USTART0   = USTART0
      int_state%PRADI_MAX = PRADI_MAX
      int_state%PMASS     = PMASS
!     
      CALL MPI_BARRIER(MPI_COMM_COMP,IRTN)
      
      IF(MYPE==0)THEN
        CLOSE(NFCST2)
      ENDIF

      IF(int_state%NUM_DUST==16)THEN
!      DO N=1,int_state%NUM_DUST
      DO N=1,8
        int_state%MN(N,1)=MN1(N)
        int_state%MN(N+8,1)=MN1(N)                         !D'Almeida emitted size distribution
        int_state%MN(N,2)=MN2(N)    
        int_state%MN(N+8,2)=MN2(N)                         !Kok emitted size distribution
        int_state%MN(N,3)=MN3(N)    
        int_state%MN(N+8,3)=MN3(N)                         !Ginoux emitted size distribution
        int_state%AA(N)=AA(N)
        int_state%AA(N+8)=AA(N)
      ENDDO
      ELSE
      DO N=1,int_state%NUM_DUST
        int_state%MN(N,1)=MN1(N)
        int_state%MN(N,2)=MN2(N)
        int_state%MN(N,3)=MN3(N)
        int_state%AA(N)=AA(N)
      ENDDO
      ENDIF
!
      DO N=1,int_state%NUM_VEG
        int_state%CDM(N)=CDM(N)
        int_state%CVM(N)=CVM(N)
        int_state%RSQCD0VM(N)=RSQCD0VM(N)
        int_state%ASM(N)=ASM(N)
      ENDDO
!           
!     
      DEALLOCATE(ASM)
      DEALLOCATE(RSQCD0VM)
      DEALLOCATE(CVM)
      DEALLOCATE(CDM)
      DEALLOCATE(AA)
      DEALLOCATE(MN1)
      DEALLOCATE(MN2)
      DEALLOCATE(MN3)
      DEALLOCATE(TEMP2)
      DEALLOCATE(TEMP3)
      DEALLOCATE(TEMP5)
      DEALLOCATE(TEMPDUST)
      DEALLOCATE(TEMPSA)
      DEALLOCATE(ITEMPDUST)
      DEALLOCATE(TEMP_PSDMAX)
      DEALLOCATE(IBIN_U)
      DEALLOCATE(IBIN_L)
      DEALLOCATE(D0)
      DEALLOCATE(DD)
      DEALLOCATE(USTART0)
      DEALLOCATE(PRADI_MAX)
      DEALLOCATE(PMASS)
      
      ENDIF dust_read           ! NUM_DUST>0
!
!
!!NOW collect or read densities and radii of each aerosol type
!from module_CONSTANTS_CHEM.F90
     IF (int_state%NUM_AERO>0)THEN
!       
        IF(int_state%NUM_ASH>0)THEN
         DO N=1,int_state%NUM_ASH 
          int_state%RHO_AERO(N) = 0.
          int_state%R_AERO_V(N) = 0.
          int_state%R_AERO_E(N) = 0.
          int_state%EPSI_AERO(N) = 0.
         ENDDO
        ENDIF !ENDIF NUM_ASH
!       
        M=int_state%NUM_ASH
!       
        IF(int_state%NUM_DUST>0)THEN
         DO N=1,int_state%NUM_DUST
          int_state%RHO_AERO(M+N) = RHO_DUST(N)
          int_state%R_AERO_V(M+N) = R_DUST_V(N)
          int_state%R_AERO_E(M+N) = R_DUST_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_DUST(N)
          if (int_state%DEXT.eq.0) then  
             int_state%QEXT550(N) = DUST_QEXT_550(N)
          elseif (int_state%DEXT.eq.1) then 
             int_state%QEXT550(N) = DUST_QEXT_550_tri(N)   
          else 
             write(0,*) 'Desired dust extinction not defined. Check DEXT in config!'
          endif 
         ENDDO
        ENDIF !ENDIF NUM_DUST
!       
        M=M+int_state%NUM_DUST
!       
        IF(int_state%NUM_SALT>0)THEN
         DO N=1,int_state%NUM_SALT
          int_state%RHO_AERO(M+N) = RHO_SALT(N)
          int_state%R_AERO_V(M+N) = R_SALT_V(N)
          int_state%R_AERO_E(M+N) = R_SALT_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_SALT(N)
         ENDDO
        ENDIF !ENDIF NUM_SALT
!       
        M=M+int_state%NUM_SALT
!
        IF(int_state%NUM_OM>0)THEN
         DO N=1,int_state%NUM_OM
          int_state%RHO_AERO(M+N) = RHO_OM(N)
          int_state%R_AERO_V(M+N) = R_OM_V(N)
          int_state%R_AERO_E(M+N) = R_OM_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_OM(N)
         ENDDO
        ENDIF !ENDIF NUM_OM
!
        M=M+int_state%NUM_OM
!
        IF(int_state%NUM_BC>0)THEN
         DO N=1,int_state%NUM_BC
          int_state%RHO_AERO(M+N) = RHO_BC(N)
          int_state%R_AERO_V(M+N) = R_BC_V(N)
          int_state%R_AERO_E(M+N) = R_BC_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_BC(N)
         ENDDO
        ENDIF !ENDIF NUM_BC
!
       M=M+int_state%NUM_BC
!
        IF(int_state%NUM_SO4>0)THEN
         DO N=1,int_state%NUM_SO4
          int_state%RHO_AERO(M+N) = RHO_SO4(N)
          int_state%R_AERO_V(M+N) = R_SO4_V(N)
          int_state%R_AERO_E(M+N) = R_SO4_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_SO4(N)
         ENDDO
        ENDIF  !ENDIF NUM_SO4
!
       M=M+int_state%NUM_SO4
!
        IF(int_state%NUM_NO3>0)THEN
         DO N=1,int_state%NUM_NO3
          int_state%RHO_AERO(M+N) = RHO_NO3(N)
          int_state%R_AERO_V(M+N) = R_NO3_V(N)
          int_state%R_AERO_E(M+N) = R_NO3_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_NO3(N)
         ENDDO
        ENDIF  !ENDIF NUM_NO3
!
       M=M+int_state%NUM_NO3
!
        IF(int_state%NUM_NH4>0)THEN
         DO N=1,int_state%NUM_NH4
          int_state%RHO_AERO(M+N) = RHO_NH4(N)
          int_state%R_AERO_V(M+N) = R_NH4_V(N)
          int_state%R_AERO_E(M+N) = R_NH4_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_NH4(N)
         ENDDO
        ENDIF  !ENDIF NUM_NH4
!
       M=M+int_state%NUM_NH4
!
        IF(int_state%NUM_UNSPC>0)THEN
         DO N=1,int_state%NUM_UNSPC
          int_state%RHO_AERO(M+N) = RHO_UNSPC(N)
          int_state%R_AERO_V(M+N) = R_UNSPC_V(N)
          int_state%R_AERO_E(M+N) = R_UNSPC_E(N)
          int_state%EPSI_AERO(M+N) = EPSI_UNSPC(N)
         ENDDO
        ENDIF  !ENDIF NUM_UNSPC
!
       M=M+int_state%NUM_UNSPC
!
        if(int_state%num_aero_uci>0)then
!         do n=1,int_state%num_aero_uci
!          rho_aero(m+n) = rho_aero_uci(n)
!          r_aero_v(m+n) = r_aero_uci_v(n)
!          r_aero_e(m+n) = r_aero_uci_e(n)
!         enddo
        endif  !endif num_aero_uci
!
      ENDIF !ENDIF NUM_AERO
!
      IF(MYPE==0)THEN
        WRITE(0,*)'**** INIT IN PHYSICS ***************'
        WRITE(0,*)' NUM_AERO =',int_state%NUM_AERO
        WRITE(0,*)' NUM_TRACERS_CHEM=',int_state%NUM_TRACERS_CHEM
        WRITE(0,*)' NUM_ASH = ',int_state%NUM_ASH
        WRITE(0,*)' NUM_DUST =',int_state%NUM_DUST
        WRITE(0,*)' NUM_SALT =',int_state%NUM_SALT
        WRITE(0,*)' NUM_OM =',int_state%NUM_OM
        WRITE(0,*)' NUM_BC=',int_state%NUM_BC
        WRITE(0,*)' NUM_SO4  =',int_state%NUM_SO4
        WRITE(0,*)' NUM_NO3  =',int_state%NUM_NO3
        WRITE(0,*)' NUM_NH4  =',int_state%NUM_NH4
        WRITE(0,*)' NUM_UNSPC=',int_state%NUM_UNSPC
        WRITE(0,*)' NUM_UCI  =',int_state%NUM_AERO_UCI 
        WRITE(0,*)' RHO_AERO =',int_state%RHO_AERO
        WRITE(0,*)' R_AERO_V  =',int_state%R_AERO_V
        WRITE(0,*)' R_AERO_E =',int_state%R_AERO_E
        WRITE(0,*)' EPSI_AERO =',int_state%EPSI_AERO
        WRITE(0,*)'*************************************'
      ENDIF
!MONARCH
!
!-----------------------------------------------------------------------
      else  read_blocks                         ! restart
!-----------------------------------------------------------------------
!
        write(infile,'(a,i2.2)')'restart_file_',my_domain_id
        open(unit=nfcst,file=infile,status='old',form='unformatted'     &
            ,iostat=ierr)
        if(ierr/=0)then
          write(0,*)' Unable to open ',trim(infile),' in READ_BINARY'
          rc = ierr
          return
        endif
!
!-----------------------------------------------------------------------
!***  Read from restart file: Integer scalars
!-----------------------------------------------------------------------
        read(nfcst) iyear_fcst
        read(nfcst) imonth_fcst
        read(nfcst) iday_fcst
        read(nfcst) ihour_fcst
        read(nfcst) !iminute_fcst
        read(nfcst) ! second_fcst
        read(nfcst) ! ntsd
        read(nfcst) ! im
        read(nfcst) ! jm
        read(nfcst) ! lm
        read(nfcst) int_state%IHRST
        read(nfcst) int_state%I_PAR_STA
        read(nfcst) int_state%J_PAR_STA
        read(nfcst) int_state%LAST_STEP_MOVED
        read(nfcst) int_state%LPT2
        read(nfcst) ! nsoil
        read(nfcst) ! nphs
        read(nfcst) ! nclod
        read(nfcst) ! nheat
        read(nfcst) ! nmts
        read(nfcst) ! nprec
        read(nfcst) ! nrdlw
        read(nfcst) ! nrdsw
        read(nfcst) ! nsrfc
!
        read(nfcst) ! cu_physics
        read(nfcst) ! mp_physics
        read(nfcst) ! lsm_physics
!
        read(nfcst) int_state%NTRACK
        read(nfcst) int_state%TRACK_HAVE_GUESS
        read(nfcst) int_state%TRACK_N_OLD
        read(nfcst) int_state%TRACKER_HAVEFIX
        read(nfcst) int_state%TRACKER_GAVE_UP
        read(nfcst) int_state%TRACKER_IFIX
        read(nfcst) int_state%TRACKER_JFIX
!
!-----------------------------------------------------------------------
!***  Read from restart file: Integer 1D arrays
!-----------------------------------------------------------------------
        read(nfcst) ! int_state%NTSCM
        read(nfcst) int_state%IDAT
!
        if(mype==0)then
          write(0,*)'**** read in core *******************'
          write(0,*)' Restart year =',iyear_fcst
          write(0,*)' Restart month=',imonth_fcst
          write(0,*)' Restart day  =',iday_fcst
          write(0,*)' Restart hour =',ihour_fcst
          write(0,*)' Original start year =',int_state%IDAT(3)
          write(0,*)' Original start month=',int_state%IDAT(2)
          write(0,*)' Original start day  =',int_state%IDAT(1)
          write(0,*)' Original start hour =',int_state%IHRST
          write(0,*)' Timestep   =',int_state%DT
          write(0,*)' Steps/hour =',3600./int_state%DT
          write(0,*)'*************************************'
        endif
!
        read(nfcst) int_state%TRACK_OLD_NTSD
!
!-----------------------------------------------------------------------
!***  Read from restart file: Real scalars
!-----------------------------------------------------------------------
        read(nfcst) ! dt
        read(nfcst) ! dyh
        read(nfcst) int_state%PDTOP
        read(nfcst) int_state%PT
        read(nfcst) int_state%TLM0D
        read(nfcst) int_state%TPH0D
        read(nfcst) ! tstart
        read(nfcst) int_state%DPHD
        read(nfcst) int_state%DLMD
        read(nfcst) int_state%SBD
        read(nfcst) int_state%WBD
!
        read(nfcst) int_state%TRACK_LAST_HOUR
        read(nfcst) int_state%TRACK_GUESS_LAT
        read(nfcst) int_state%TRACK_GUESS_LON
        read(nfcst) int_state%TRACK_EDGE_DIST
        read(nfcst) int_state%TRACK_STDERR_M1
        read(nfcst) int_state%TRACK_STDERR_M2
        read(nfcst) int_state%TRACK_STDERR_M3
        read(nfcst) int_state%TRACKER_FIXLAT
        read(nfcst) int_state%TRACKER_FIXLON
        read(nfcst) int_state%TRACKER_RMW
        read(nfcst) int_state%TRACKER_PMIN
        read(nfcst) int_state%TRACKER_VMAX
!-----------------------------------------------------------------------
!***  Read from restart file: Real 1D arrays
!-----------------------------------------------------------------------
        read(nfcst) ! dxh
        read(nfcst) int_state%SG1
        read(nfcst) int_state%SG2
        read(nfcst) int_state%DSG1
        read(nfcst) int_state%DSG2
        read(nfcst) int_state%SGML1
        read(nfcst) int_state%SGML2
        read(nfcst) int_state%SGM
        read(nfcst) int_state%EPSL
        read(nfcst) int_state%EPSQ2
        read(nfcst) int_state%SLDPTH
        read(nfcst) int_state%MP_RESTART_STATE
        read(nfcst) int_state%TBPVS_STATE
        read(nfcst) int_state%TBPVS0_STATE
!
        read(nfcst) int_state%TRACK_OLD_LAT
        read(nfcst) int_state%TRACK_OLD_LON
!
        DO L=1,LM
          int_state%PDSG1(L)=int_state%DSG1(L)*int_state%PDTOP
          int_state%PSGML1(L)=int_state%SGML1(L)*int_state%PDTOP+int_state%PT
        ENDDO
!
        DO L=1,LM+1
          int_state%PSG1(L)=int_state%SG1(L)*int_state%PDTOP+int_state%PT
        ENDDO
!
!-----------------------------------------------------------------------
!***  Read in the full-domain 1-D datastring of boundary winds.
!***  Each task isolates its own piece of that data.
!-----------------------------------------------------------------------
!
        length=(int_state%nlev_h*int_state%lnsh                         &  !<-- Total # of words
               +int_state%nlev_v*int_state%lnsv)                        &  !    in full-domain
               *2*2*(ide-ids+jde-jds+2)                                    !    boundary arrays.
        allocate(all_bc_data(1:length))
!
        read(nfcst) all_bc_data
!
!-----------------------------------------------------------------------
!
        nvars_bc_2d_h=int_state%nvars_bc_2d_h
        nvars_bc_3d_h=int_state%nvars_bc_3d_h
        nvars_bc_4d_h=int_state%nvars_bc_4d_h
        nvars_bc_2d_v=int_state%nvars_bc_2d_v
        nvars_bc_3d_v=int_state%nvars_bc_3d_v
!
        kount=0
!
!-----------------------------------------------------------------------
!
        iend=min(ite_h2,ide)
!
        if(nvars_bc_2d_h>0)then
          do nv=1,nvars_bc_2d_h
            do n=1,2
            do j=1,int_state%lnsh
            do i=ids,ide
              kount=kount+1
              if(jts==jds.and.i>=its_h2.and.i<=iend)then                   !<-- South boundary tasks extract 2-D BC H-pt data
                int_state%bnd_vars_h%var_2d(nv)%south(i,j,n)=           &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_h>0)then
          do nv=1,nvars_bc_3d_h
            do n=1,2
            do l=1,lm
            do j=1,int_state%lnsh
            do i=ids,ide
              kount=kount+1
              if(jts==jds.and.i>=its_h2.and.i<=iend)then                   !<-- South boundary tasks extract 3-D BC H-pt data
                int_state%bnd_vars_h%var_3d(nv)%south(i,j,l,n)=         &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_4d_h>0)then
          do nv=1,nvars_bc_4d_h
            lb=int_state%lbnd_4d(nv)
            ub=int_state%ubnd_4d(nv)
            do nl=lb,ub
            do n=1,2
            do l=1,lm
            do j=1,int_state%lnsh
            do i=ids,ide
              kount=kount+1
              if(jts==jds.and.i>=its_h2.and.i<=iend)then                   !<-- South boundary tasks extract 4-D BC H-pt data
                int_state%bnd_vars_h%var_4d(nv)%south(i,j,l,n,nl)=      &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_2d_v>0)then
          do nv=1,nvars_bc_2d_v
            do n=1,2
            do j=1,int_state%lnsv
            do i=ids,ide
              kount=kount+1
              if(jts==jds.and.i>=its_h2.and.i<=iend)then                   !<-- South boundary tasks extract 2-D BC V-pt data
                int_state%bnd_vars_v%var_2d(nv)%south(i,j,n)=           &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_v>0)then
          do nv=1,nvars_bc_3d_v
            do n=1,2
            do l=1,lm
            do j=1,int_state%lnsv
            do i=ids,ide
              kount=kount+1
              if(jts==jds.and.i>=its_h2.and.i<=iend)then                   !<-- South boundary tasks extract 3-D BC V-pt data
                int_state%bnd_vars_v%var_3d(nv)%south(i,j,l,n)=         &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_2d_h>0)then
          do nv=1,nvars_bc_2d_h
            do n=1,2
            do j=1,int_state%lnsh
            do i=ids,ide
              kount=kount+1
              if(jte==jde.and.i>=its_h2.and.i<=iend)then                   !<-- North boundary tasks extract 2-D BC H-pt data
                int_state%bnd_vars_h%var_2d(nv)%north(i,j,n)=           &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_h>0)then
          do nv=1,nvars_bc_3d_h
            do n=1,2
            do l=1,lm
            do j=1,int_state%lnsh
            do i=ids,ide
              kount=kount+1
              if(jte==jde.and.i>=its_h2.and.i<=iend)then                   !<-- North boundary tasks extract 3-D BC H-pt data
                int_state%bnd_vars_h%var_3d(nv)%north(i,j,l,n)=         &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_4d_h>0)then
          do nv=1,nvars_bc_4d_h
            lb=int_state%lbnd_4d(nv)
            ub=int_state%ubnd_4d(nv)
            do nl=lb,ub
            do n=1,2
            do l=1,lm
            do j=1,int_state%lnsh
            do i=ids,ide
              kount=kount+1
              if(jte==jde.and.i>=its_h2.and.i<=iend)then                   !<-- North boundary tasks extract 4-D BC H-pt data
                int_state%bnd_vars_h%var_4d(nv)%north(i,j,l,n,nl)=      &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_2d_v>0)then
          do nv=1,nvars_bc_2d_v
            do n=1,2
            do j=1,int_state%lnsv
            do i=ids,ide
              kount=kount+1
              if(jte==jde.and.i>=its_h2.and.i<=iend)then                   !<-- North boundary tasks extract 2-D BC V-pt data
                int_state%bnd_vars_v%var_2d(nv)%north(i,j,n)=           &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_v>0)then
          do nv=1,nvars_bc_3d_v
            do n=1,2
            do l=1,lm
            do j=1,int_state%lnsv
            do i=ids,ide
              kount=kount+1
              if(jte==jde.and.i>=its_h2.and.i<=iend)then                   !<-- North boundary tasks extract 3-D BC V-pt data
                int_state%bnd_vars_v%var_3d(nv)%north(i,j,l,n)=         &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        jend=min(jte_h2,jde)
!
        if(nvars_bc_2d_h>0)then
          do nv=1,nvars_bc_2d_h
            do n=1,2
            do j=jds,jde
            do i=1,int_state%lnsh
              kount=kount+1
              if(its==ids.and.j>=jts_h2.and.j<=jend)then                   !<-- West boundary tasks extract 2-D BC H-pt data
                int_state%bnd_vars_h%var_2d(nv)%west(i,j,n)=            &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_h>0)then
          do nv=1,nvars_bc_3d_h
            do n=1,2
            do l=1,lm
            do j=jds,jde
            do i=1,int_state%lnsh
              kount=kount+1
              if(its==ids.and.j>=jts_h2.and.j<=jend)then                   !<-- West boundary tasks extract 3-D BC H-pt data
                int_state%bnd_vars_h%var_3d(nv)%west(i,j,l,n)=          &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_4d_h>0)then
          do nv=1,nvars_bc_4d_h
            lb=int_state%lbnd_4d(nv)
            ub=int_state%ubnd_4d(nv)
            do nl=lb,ub
            do n=1,2
            do l=1,lm
            do j=jds,jde
            do i=1,int_state%lnsh
              kount=kount+1
              if(its==ids.and.j>=jts_h2.and.j<=jend)then                   !<-- West boundary tasks extract 4-D BC H-pt data
                int_state%bnd_vars_h%var_4d(nv)%west(i,j,l,n,nl)=       &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_2d_v>0)then
          do nv=1,nvars_bc_2d_v
            do n=1,2
            do j=jds,jde
            do i=1,int_state%lnsv
              kount=kount+1
              if(its==ids.and.j>=jts_h2.and.j<=jend)then                   !<-- West boundary tasks extract 2-D BC V-pt data
                int_state%bnd_vars_v%var_2d(nv)%west(i,j,n)=            &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_v>0)then
          do nv=1,nvars_bc_3d_v
            do n=1,2
            do l=1,lm
            do j=jds,jde
            do i=1,int_state%lnsv
              kount=kount+1
              if(its==ids.and.j>=jts_h2.and.j<=jend)then                   !<-- West boundary tasks extract 3-D BC V-pt data
                int_state%bnd_vars_v%var_3d(nv)%west(i,j,l,n)=          &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_2d_h>0)then
          do nv=1,nvars_bc_2d_h
            do n=1,2
            do j=jds,jde
            do i=1,int_state%lnsh
              kount=kount+1
              if(ite==ide.and.j>=jts_h2.and.j<=jend)then                   !<-- East boundary tasks extract 2-D BC H-pt data
                int_state%bnd_vars_h%var_2d(nv)%east(i,j,n)=            &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_h>0)then
          do nv=1,nvars_bc_3d_h
            do n=1,2
            do l=1,lm
            do j=jds,jde
            do i=1,int_state%lnsh
              kount=kount+1
              if(ite==ide.and.j>=jts_h2.and.j<=jend)then                   !<-- East boundary tasks extract 3-D BC H-pt data
                int_state%bnd_vars_h%var_3d(nv)%east(i,j,l,n)=          &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_4d_h>0)then
          do nv=1,nvars_bc_4d_h
            lb=int_state%lbnd_4d(nv)
            ub=int_state%ubnd_4d(nv)
            do nl=lb,ub
            do n=1,2
            do l=1,lm
            do j=jds,jde
            do i=1,int_state%lnsh
              kount=kount+1
              if(ite==ide.and.j>=jts_h2.and.j<=jend)then                   !<-- East boundary tasks extract 4-D BC H-pt data
                int_state%bnd_vars_h%var_4d(nv)%east(i,j,l,n,nl)=       &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_2d_v>0)then
          do nv=1,nvars_bc_2d_v
            do n=1,2
            do j=jds,jde
            do i=1,int_state%lnsv
              kount=kount+1
              if(ite==ide.and.j>=jts_h2.and.j<=jend)then                   !<-- East boundary tasks extract 2-D BC V-pt data
                int_state%bnd_vars_v%var_2d(nv)%east(i,j,n)=            &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
          enddo
        endif
!
        if(nvars_bc_3d_v>0)then
          do nv=1,nvars_bc_3d_v
            do n=1,2
            do l=1,lm
            do j=jds,jde
            do i=1,int_state%lnsv
              kount=kount+1
              if(ite==ide.and.j>=jts_h2.and.j<=jend)then                   !<-- East boundary tasks extract 3-D BC V-pt data
                int_state%bnd_vars_v%var_3d(nv)%east(i,j,l,n)=          &
                  all_bc_data(kount)
              endif
            enddo
            enddo
            enddo
            enddo
          enddo
        endif
!
        deallocate(all_bc_data)
!
!-----------------------------------------------------------------------
!***  Read from restart file: Logical
!-----------------------------------------------------------------------
        read(nfcst) ! global
        read(nfcst) int_state%RUN
        read(nfcst) ! adiabatic
!
!-----------------------------------------------------------------------
!***  Read from restart file: Integer 2D arrays
!-----------------------------------------------------------------------
!
      ALLOCATE(ITEMP(IDS:IDE,JDS:JDE),STAT=I)
!
      IF(MYPE==0)THEN
        READ(NFCST) ITEMP
      ENDIF
      CALL IDSTRB(ITEMP,int_state%ISLTYP,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST) ITEMP
      ENDIF
      CALL IDSTRB(ITEMP,int_state%IVGTYP,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST) ITEMP
      ENDIF
      CALL IDSTRB(ITEMP,int_state%NCFRCV,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST) ITEMP
      ENDIF
      CALL IDSTRB(ITEMP,int_state%NCFRST,MYPE,MPI_COMM_COMP)
!
      IF(MYPE==0)THEN
        READ(NFCST) ITEMP
      ENDIF
      CALL IDSTRB(ITEMP,int_state%TRACKER_FIXES,MYPE,MPI_COMM_COMP)
!
      DEALLOCATE(ITEMP)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!***  Read from restart file by alphabetical order (new in ESMF6)
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!***  ACFRCV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACFRCV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ACFRST
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACFRST,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ACPREC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACPREC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ACPREC_TOT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACPREC_TOT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ACSNOM
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACSNOM,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ACSNOW
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACSNOW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ACUTIM
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ACUTIM,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  AKHS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%AKHS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  AKHS_OUT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%AKHS_OUT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  AKMS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%AKMS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  AKMS_OUT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%AKMS_OUT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ALBASE
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ALBASE,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ALBEDO
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ALBEDO,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ALWIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ALWIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ALWOUT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ALWOUT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ALWTOA
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ALWTOA,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  APHTIM
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%APHTIM,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ARDLW
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ARDLW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ARDSW
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ARDSW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ASRFC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ASRFC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ASWIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ASWIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ASWOUT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ASWOUT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  ASWTOA
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%ASWTOA,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  AVCNVC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%AVCNVC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  AVRAIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%AVRAIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  BGROFF
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%BGROFF,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CFRACH
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CFRACH,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CFRACL
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CFRACL,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CFRACM
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CFRACM,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CLDEFI
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CLDEFI,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CLDFRA
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,INT_STATE%CLDFRA,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  CMC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CMC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CNVBOT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CNVBOT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CNVTOP
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CNVTOP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CPRATE
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CPRATE,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CUPPT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CUPPT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CUPREC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CUPREC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CW
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%CW(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%CW,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%CW,LM,2,2)
!-----------------------------------------------------------------------
!***  CZEN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CZEN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  CZMEAN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%CZMEAN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  DFI_TTEN
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%DFI_TTEN,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%DFI_TTEN,LM,2,2)
!-----------------------------------------------------------------------
!***  DIV
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%DIV,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%DIV,LM,2,2)
!-----------------------------------------------------------------------
!***  DWDT
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%DWDT(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%DWDT,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%DWDT,LM,2,2)
!-----------------------------------------------------------------------
!***  EPSR
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%EPSR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  F
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%F(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%F,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%F,1,2,2)
!-----------------------------------------------------------------------
!***  FIS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%FIS(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%FIS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%FIS,1,2,2)
!-----------------------------------------------------------------------
!***  F_ICE
!-----------------------------------------------------------------------
      DO K=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%F_ICE(I,J,K)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%F_ICE,1,1,1,LM,K,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  F_RAIN
!-----------------------------------------------------------------------
      DO K=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%F_RAIN(I,J,K)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%F_RAIN,1,1,1,LM,K,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  F_RIMEF
!-----------------------------------------------------------------------
      DO K=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%F_RIMEF(I,J,K)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%F_RIMEF,1,1,1,LM,K,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  GLAT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%GLAT(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%GLAT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%GLAT,1,2,2)
!-----------------------------------------------------------------------
!***  GLON
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%GLON(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%GLON,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%GLON,1,2,2)
!-----------------------------------------------------------------------
!***  GRNFLX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%GRNFLX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  HBOT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%HBOT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  HBOTD
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%HBOTD,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  HBOTS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%HBOTS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  HDACVX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%HDACVX(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%HDACVX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%HDACVX,1,2,2)
!-----------------------------------------------------------------------
!***  HDACVY
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%HDACVY(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%HDACVY,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%HDACVY,1,2,2)
!-----------------------------------------------------------------------
!***  HDACX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%HDACX(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%HDACX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%HDACX,1,2,2)
!-----------------------------------------------------------------------
!***  HDACY
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%HDACY(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%HDACY,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%HDACY,1,2,2)
!-----------------------------------------------------------------------
!***  HTOP
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%HTOP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  HTOPD
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%HTOPD,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  HTOPS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%HTOPS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  M10RV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%M10RV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  M10WIND
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%M10WIND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  MEMBRANE_MSLP
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%MEMBRANE_MSLP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  MXSNAL (SNOW ALBEDO)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%MXSNAL,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  O3
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%O3,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%O3,LM,2,2)
!-----------------------------------------------------------------------
!***  OMGALF
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%OMGALF,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%OMGALF,LM,2,2)
!-----------------------------------------------------------------------
!***  P500U
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P500U,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P500V
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P500V,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P700RV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P700RV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P700U
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P700U,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P700V
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P700V,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P700WIND
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P700WIND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P700Z
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P700Z,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P850RV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P850RV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P850U
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P850U,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P850V
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P850V,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P850WIND
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P850WIND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  P850Z
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%P850Z,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  PBLH
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%PBLH,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  PD
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%PD(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%PD,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%PD,1,2,2)
!-----------------------------------------------------------------------
!***  PDO
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%PDO(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%PDO,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%PDO,1,2,2)
!-----------------------------------------------------------------------
!***  PINT
!-----------------------------------------------------------------------
      DO L=1,LM+1
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%PINT(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%PINT,1,1,1,LM+1,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%PINT,LM+1,2,2)
!-----------------------------------------------------------------------
!***  POTEVP
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%POTEVP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  POTFLX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%POTFLX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  PREC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%PREC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  PSGDT
!-----------------------------------------------------------------------
      DO L=1,LM-1
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
      ENDDO
!-----------------------------------------------------------------------
!***  PSHLTR
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%PSHLTR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  Q10
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%Q10,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  Q2
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%Q2(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%Q2,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%Q2,LM,2,2)
!-----------------------------------------------------------------------
!***  QSH
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%QSH,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  QSHLTR
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%QSHLTR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  QWBS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%QWBS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  QZ0
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%QZ0,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  Q
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%Q(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%Q,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%Q,LM,2,2)
!-----------------------------------------------------------------------
!***  RADOT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RADOT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RLWIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RLWIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RLWTOA
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RLWTOA,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RLWTT
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%RLWTT(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%RLWTT,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  RMOL
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RMOL,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RSWIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RSWIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RSWINC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RSWINC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RSWOUT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RSWOUT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RSWTOA
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%RSWTOA,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  RSWTT
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%RSWTT(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%RSWTT,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  SFCEVP
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SFCEVP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SFCEXC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SFCEXC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SFCLHX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SFCLHX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SFCSHX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SFCSHX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SH2O
!-----------------------------------------------------------------------
      DO K=1,int_state%NSOIL
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%SH2O,1,1,1,int_state%NSOIL,K        &
                  ,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  SI
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SI,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SICE
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SICE,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SIGT4
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SIGT4,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SM (Seamask)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%SM(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%SM,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SM10RV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SM10RV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SM10WIND
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SM10WIND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SMC
!-----------------------------------------------------------------------
      DO K=1,int_state%NSOIL
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%SMC,1,1,1,int_state%NSOIL,K        &
                  ,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  SMSLP
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SMSLP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SMSTAV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SMSTAV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SMSTOT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SMSTOT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SNO
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SNO,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SNOPCX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SNOPCX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SNOWC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SNOWC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SOILTB
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SOILTB,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SP700RV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SP700RV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SP700WIND
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SP700WIND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SP700Z
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SP700Z,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SP850RV
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SP850RV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SP850WIND
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SP850WIND,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SP850Z
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SP850Z,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SR
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SSROFF
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SSROFF,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  SST
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SST,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  STC
!-----------------------------------------------------------------------
      DO K=1,int_state%NSOIL
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%STC,1,1,1,int_state%NSOIL,K          &
                  ,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  SUBSHX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%SUBSHX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  T2
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%T2,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TCT
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%TCT,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%TCT,LM,2,2)
!-----------------------------------------------------------------------
!***  TCUCN
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%TCUCN(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%TCUCN,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  TCU
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%TCU,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%TCU,LM,2,2)
!-----------------------------------------------------------------------
!***  TCV
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%tcv,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%TCV,LM,2,2)
!-----------------------------------------------------------------------
!***  TG
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TG,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TH10
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TH10,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  THS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%THS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  THZ0
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%THZ0,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TLMAX
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TLMAX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TLMIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TLMIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TP
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%TP,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%TP,LM,2,2)
!-----------------------------------------------------------------------
!***  TRACERS
!-----------------------------------------------------------------------
      DO N=int_state%INDX_Q2+1,int_state%NUM_TRACERS_TOTAL  !<-- The first 'indx_q2' arrays are unallocated pointers
        DO L=1,LM
          IF(MYPE==0)THEN
            READ(NFCST)TEMP1
          ENDIF
          DO J=JMS,JME
          DO I=IMS,IME
            int_state%TRACERS(I,J,L,N)=0.
          ENDDO
          ENDDO
          CALL DSTRB(TEMP1,int_state%TRACERS(:,:,:,N),1,1,1,LM,L      &
                    ,MYPE,MPI_COMM_COMP)
        ENDDO
      ENDDO
      CALL HALO_EXCH(int_state%TRACERS,LM,int_state%NUM_TRACERS_TOTAL,1,2,2)
!-----------------------------------------------------------------------
!***  TRACERS_PREV
!-----------------------------------------------------------------------
      DO N=1,int_state%NUM_TRACERS_TOTAL
        DO L=1,LM
          IF(MYPE==0)THEN
            READ(NFCST)TEMP1
          ENDIF
          CALL DSTRB(TEMP1,int_state%TRACERS_PREV(:,:,:,N),1,1,1,LM,L &
                    ,MYPE,MPI_COMM_COMP)
        ENDDO
      ENDDO
      CALL HALO_EXCH(int_state%TRACERS_PREV,LM,int_state%NUM_TRACERS_TOTAL,1,2,2)
!-----------------------------------------------------------------------
!***  TRACKER_ANGLE
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TRACKER_ANGLE,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TRACKER_DISTSQ
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TRACKER_DISTSQ,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TRAIN
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%TRAIN(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%TRAIN,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  TSHLTR
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TSHLTR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TSKIN
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TSKIN,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  TWBS
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%TWBS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  T
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
          WRITE(0,*) 'L, T extremes: ', L, MINVAL(TEMP1),MAXVAL(TEMP1)
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%T(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%T,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%T,LM,2,2)
!-----------------------------------------------------------------------
!***  TADJ
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%TADJ(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%TADJ,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%TADJ,LM,2,2)
!-----------------------------------------------------------------------
!***  TOLD
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%TOLD(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%TOLD,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%TOLD,LM,2,2)
!-----------------------------------------------------------------------
!***  U10
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%U10,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  UP
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%UP,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%UP,LM,2,2)
!-----------------------------------------------------------------------
!***  USTAR
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%USTAR,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  UZ0
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%UZ0,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  U
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%U(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%U,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%U,LM,2,2)
!-----------------------------------------------------------------------
!***  V10
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%V10,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  VEGFRC
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST) TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%VEGFRC,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  VLAT
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%VLAT(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%VLAT,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%VLAT,1,2,2)
!-----------------------------------------------------------------------
!***  VLON
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      DO J=JMS,JME
      DO I=IMS,IME
        int_state%VLON(I,J)=0.
      ENDDO
      ENDDO
      CALL DSTRB(TEMP1,int_state%VLON,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%VLON,1,2,2)
!-----------------------------------------------------------------------
!***  VP
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%VP,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%VP,LM,2,2)
!-----------------------------------------------------------------------
!***  VZ0
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST) TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%VZ0,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  V
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%V(I,J,L)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%V,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%V,LM,2,2)
!-----------------------------------------------------------------------
!***  W
!-----------------------------------------------------------------------
      int_state%W=0.0
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%W,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%W,LM,2,2)
!-----------------------------------------------------------------------
!***  XLEN_MIX
!-----------------------------------------------------------------------
      DO K=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        DO J=JMS,JME
        DO I=IMS,IME
          int_state%XLEN_MIX(I,J,K)=0.
        ENDDO
        ENDDO
        CALL DSTRB(TEMP1,int_state%XLEN_MIX,1,1,1,LM,K,MYPE,MPI_COMM_COMP)
      ENDDO
!-----------------------------------------------------------------------
!***  Z0
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%Z0,1,1,1,1,1,MYPE,MPI_COMM_COMP)
      CALL HALO_EXCH(int_state%Z0,1,3,3)
!-----------------------------------------------------------------------
!***  Z0BASE
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NFCST)TEMP1
      ENDIF
      CALL DSTRB(TEMP1,int_state%Z0BASE,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!***  Z
!-----------------------------------------------------------------------
      DO L=1,LM
        IF(MYPE==0)THEN
          READ(NFCST)TEMP1
        ENDIF
        CALL DSTRB(TEMP1,int_state%Z,1,1,1,LM,L,MYPE,MPI_COMM_COMP)
      ENDDO
      CALL HALO_EXCH(int_state%Z,LM,2,2)
!-----------------------------------------------------------------------
!
        close(nfcst)
!
!-----------------------------------------------------------------------
!
        tend_max=real(int_state%IHREND)
        ntstm_max=nint(tend_max*3600./int_state%DT)+1
        tend=real(int_state%nhours_fcst)
        int_state%NTSTM=nint(tend*3600./int_state%DT)+1
        if(.not.int_state%GLOBAL)then
          if(mype==0)then
            write(0,*)' Max runtime is ',tend_max,' hours'
          endif
        endif
        if(mype==0)then
          write(0,*)' Requested runtime is ',tend,' hours'
          write(0,*)' NTSTM=',int_state%NTSTM
        endif
        if(int_state%NTSTM>ntstm_max.and..not.int_state%GLOBAL)then
          if(mype==0)then
            write(0,*)' Requested fcst length exceeds maximum'
            write(0,*)' Resetting to maximum'
          endif
          int_state%NTSTM=min(int_state%NTSTM,ntstm_max)
        endif
!
        ntsd=0
        int_state%IHR=nint(ntsd*int_state%DT/3600.)
!-----------------------------------------------------------------------
        do l=1,lm
          int_state%PDSG1(l)=int_state%DSG1(l)*int_state%PDTOP
          int_state%PSGML1(l)=int_state%SGML1(l)*int_state%PDTOP+int_state%PT
        enddo
!
        do l=1,lm+1
          int_state%PSG1(l)=int_state%SG1(l)*int_state%PDTOP+int_state%PT
        enddo
!-----------------------------------------------------------------------
      endif  read_blocks                        ! cold start /restart
!-----------------------------------------------------------------------
!
      deallocate(temp1)
!
!-----------------------------------------------------------------------
!
      end subroutine read_binary
!
!-----------------------------------------------------------------------
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!-----------------------------------------------------------------------
!
      SUBROUTINE PHYSICS_READ_GWD(INFILE,NGWD,INT_STATE                &
                                 ,MYPE,MPI_COMM_COMP                   &
                                 ,IDS,IDE,JDS,JDE,RC)
!----------------------------------------------------------------------
!
!------------------------
!***  Argument variables
!------------------------
!
      INTEGER,INTENT(IN) :: NGWD,MYPE,MPI_COMM_COMP
      INTEGER,INTENT(IN) :: IDS,IDE,JDS,JDE
!
      CHARACTER(LEN=*),INTENT(IN) :: INFILE
!
      TYPE(SOLVER_INTERNAL_STATE),POINTER,INTENT(INOUT) :: INT_STATE     !<-- The physics internal state
!
      INTEGER,INTENT(OUT) :: RC
!
!---------------------
!***  Local variables
!---------------------
!
      INTEGER :: IERR
!
      REAL,DIMENSION(:,:),ALLOCATABLE :: TEMP_GWD
!
!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
!
      RC = 0
!
      ALLOCATE(TEMP_GWD(IDS:IDE,JDS:JDE))      
!
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        OPEN(unit=NGWD,file=INFILE,status='old',form='unformatted'      &
            ,iostat=IERR)
        IF(IERR/=0)THEN
          WRITE(0,*)' Unable to open file ',TRIM(INFILE)                &
                   ,' in PHYSICS_READ_GWD'
          RC = IERR
          RETURN
        ENDIF
      ENDIF
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HSTDV,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HCNVX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HASYW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HASYS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HASYSW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HASYNW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HLENW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HLENS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HLENSW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HLENNW,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HANGL,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HANIS,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HSLOP,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
      IF(MYPE==0)THEN
        READ(NGWD)TEMP_GWD
      ENDIF
!
      CALL DSTRB(TEMP_GWD,int_state%HZMAX,1,1,1,1,1,MYPE,MPI_COMM_COMP)
!-----------------------------------------------------------------------
!
      IF(MYPE==0)THEN
        CLOSE(NGWD)
      ENDIF
!
      DEALLOCATE(TEMP_GWD)
!-----------------------------------------------------------------------
!
      END SUBROUTINE PHYSICS_READ_GWD 
!
!-----------------------------------------------------------------------
!***********************************************************************
      subroutine check_stat(stat,message)
      use netcdf
      implicit none
      include 'netcdf.inc'

      integer, intent(in)       :: stat
      character*100, intent(in) :: message

      if ( stat /= nf90_NoErr ) then
        print*, 'error in netcdf status while...'
        print*, TRIM(message)
        STOP
      endif

      end subroutine check_stat
!#######################################################################
!-----------------------------------------------------------------------
!
      end module module_INIT_READ_BIN
!
!-----------------------------------------------------------------------
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!-----------------------------------------------------------------------

