MODULE module_data_mgn2mech
! WRF-Chem MEGAN v2.04 code adapted for MONARCH

!WRF was developed at the National Center for Atmospheric Research (NCAR)
!which is operated by the University Corporation for Atmospheric Research
!(UCAR). NCAR and UCAR make no proprietary claims, either statutory or
!otherwise, to this version and release of WRF and consider WRF to be in
!the public domain for use by any person or entity for any purpose without
!any fee or charge. UCAR requests that any WRF user include this notice on
!any partial or full copies of WRF. WRF is provided on an "AS IS" basis and
!any warranties, either express or implied, including but not limited to
!implied warranties of non-infringement, originality, merchantability and
!fitness for a particular purpose, are disclaimed. In no event shall UCAR be
!liable for any damages, whatsoever, whether direct, indirect, consequential
!or special, that arise out of or in connection with the access, use or
!performance of WRF, including infringement actions.

  ! This module contains speciation to convert MEGAN emissions
  ! species to various gas-phase mechanism species
  !
  ! August, 2007
  !
  ! Serena H. Chung          Washington State University
  ! Tan Sakulyanontvittaya   University of Colorado
  ! Christine Wiedinmyer     National Center for Atmospheric Research
 


  !
  ! provides p_iso, p_par, etc 
  USE MODULE_BSC_CHEM_DATA, ONLY :     &
                           P_NO        &
                          ,P_FORM      &
                          ,P_ALD2      &
                          ,P_ALDX      &
                          ,P_PAR       &
                          ,P_CO        &
                          ,P_MEOH      &
                          ,P_OLE       &
                          ,P_ETH       &
                          ,P_IOLE      &
                          ,P_TOL       &
                          ,P_ISOP      &
                          ,P_TERP      &
                          ,P_ETOH      &
                          ,P_ETHA      &
                          ,P_PINENE_A  &
                          ,P_ISOP
  ! provides is_soprene, is_myrcene, etc
  USE module_data_megan2        

 
  IMPLICIT NONE
  SAVE

 ! for MEGAN species not in the reaction mechanisms
  INTEGER, PARAMETER :: non_react = 9999

  INTEGER, PARAMETER :: n_megan2cbm05 = 173
  INTEGER, DIMENSION (n_megan2cbm05) :: p_of_megan2cbm05, p_of_cbm05
  REAL,    DIMENSION (n_megan2cbm05) :: cbm05_per_megan
  DATA p_of_cbm05 / n_megan2cbm05*non_react /






 
  !--------------------------------------------------------------------
  ! Some naming convention in denoting MEGAN species
  ! _a  = alpha     , _b  = beta      , _c  = cis       , _al = allo    ,
  ! _g  = gamma     , _d  = delta     , _t  = trans     , _m  = methyl  ,
  ! _p  = para      , _o  = ortho     , _e  = ene       , _ol = ol      ,
  ! met = methyl    , 2met= dimethyl  , MBO = methylbutenol             ,
  ! 2s  = disulfide , s   = sulfide   , OXD = oxide     , ACT = acetate ,
  ! PPPP= propenylpropyl              , DCTT= decatetraene              ,
  ! CCO = acetaldehyde


CONTAINS


  !--------------------------------------------------------------------


    ! For MEGAN v2.04 species conversion to CBMZ species 
    ! Based on Tan's MAP_CV2CBMZ.EXT; updated on 08/03/2007
    ! based on Rahul A. Zaveri's suggestions.


  !--------------------------------------------------------------------


    ! For MEGAN v2.04 species conversion to RADM2 species 
    ! Based on Tan's MAP_CV2RADM2.EXT

    
  !--------------------------------------------------------------------


    ! For MEGAN v2.04 species conversion to RACM species 
    ! Based on Tan's MAP_CV2RACM.EXT

    !

  !--------------------------------------------------------------------

  SUBROUTINE get_megan2cbm05_table

    ! For MEGAN v2.04 species conversion to cbm05 species 
    ! Based on Tan's MAP_CV2cbm05.EXT; updated on 08/03/2007
    ! based on Rahul A. Zaveri's suggestions.


    !
    ! Index of                                       Index of                      Molar ratio
    ! MEGAN species                                  CMB05 Species                             
    !
    p_of_megan2cbm05(  1) = is_isoprene             ; p_of_cbm05(  1) = p_isop      ; cbm05_per_megan(  1)  =  1.  
    p_of_megan2cbm05(  2) = is_myrcene              ; p_of_cbm05(  2) = p_terp      ; cbm05_per_megan(  2)  =  1.  
    p_of_megan2cbm05(  3) = is_sabinene             ; p_of_cbm05(  3) = p_terp      ; cbm05_per_megan(  3)  =  1.  
    p_of_megan2cbm05(  4) = is_limonene             ; p_of_cbm05(  4) = p_terp      ; cbm05_per_megan(  4)  =  1.  
    p_of_megan2cbm05(  5) = is_carene_3             ; p_of_cbm05(  5) = p_terp      ; cbm05_per_megan(  5)  =  1.  
    p_of_megan2cbm05(  6) = is_ocimene_t_b          ; p_of_cbm05(  6) = p_terp      ; cbm05_per_megan(  6)  =  1.  
    p_of_megan2cbm05(  7) = is_pinene_b             ; p_of_cbm05(  7) = p_terp      ; cbm05_per_megan(  7)  =  1.  
    p_of_megan2cbm05(  8) = is_pinene_a             ; p_of_cbm05(  8) = p_terp      ; cbm05_per_megan(  8)  =  1.  
    p_of_megan2cbm05(  9) = is_2met_styrene         ; p_of_cbm05(  9) = p_terp      ; cbm05_per_megan(  9)  =  1.  
    p_of_megan2cbm05( 10) = is_cymene_p             ; p_of_cbm05( 10) = p_terp      ; cbm05_per_megan( 10)  =  1.  
    p_of_megan2cbm05( 11) = is_cymene_o             ; p_of_cbm05( 11) = p_terp      ; cbm05_per_megan( 11)  =  1.  
    p_of_megan2cbm05( 12) = is_phellandrene_a       ; p_of_cbm05( 12) = p_terp      ; cbm05_per_megan( 12)  =  1.  
    p_of_megan2cbm05( 13) = is_thujene_a            ; p_of_cbm05( 13) = p_terp      ; cbm05_per_megan( 13)  =  1.  
    p_of_megan2cbm05( 14) = is_terpinene_a          ; p_of_cbm05( 14) = p_terp      ; cbm05_per_megan( 14)  =  1.  
    p_of_megan2cbm05( 15) = is_terpinene_g          ; p_of_cbm05( 15) = p_terp      ; cbm05_per_megan( 15)  =  1.  
    p_of_megan2cbm05( 16) = is_terpinolene          ; p_of_cbm05( 16) = p_terp      ; cbm05_per_megan( 16)  =  1. 
    p_of_megan2cbm05( 17) = is_phellandrene_b       ; p_of_cbm05( 17) = p_terp      ; cbm05_per_megan( 17)  =  1.  
    p_of_megan2cbm05( 18) = is_camphene             ; p_of_cbm05( 18) = p_terp      ; cbm05_per_megan( 18)  =  1.  
    p_of_megan2cbm05( 19) = is_bornene              ; p_of_cbm05( 19) = p_terp      ; cbm05_per_megan( 19)  =  1.  
    p_of_megan2cbm05( 20) = is_fenchene_a           ; p_of_cbm05( 20) = p_terp      ; cbm05_per_megan( 20)  =  1.  
    p_of_megan2cbm05( 21) = is_ocimene_al           ; p_of_cbm05( 21) = p_terp      ; cbm05_per_megan( 21)  =  1.  
    p_of_megan2cbm05( 22) = is_ocimene_c_b          ; p_of_cbm05( 22) = p_terp      ; cbm05_per_megan( 22)  =  1.  
    p_of_megan2cbm05( 23) = is_tricyclene           ; p_of_cbm05( 23) = p_terp      ; cbm05_per_megan( 23)  =  1.  
    p_of_megan2cbm05( 24) = is_estragole            ; p_of_cbm05( 24) = p_terp      ; cbm05_per_megan( 24)  =  1.  
    p_of_megan2cbm05( 25) = is_camphor              ; p_of_cbm05( 25) = p_terp      ; cbm05_per_megan( 25)  =  1.  
    p_of_megan2cbm05( 26) = is_fenchone             ; p_of_cbm05( 26) = p_terp      ; cbm05_per_megan( 26)  =  1.  
    p_of_megan2cbm05( 27) = is_piperitone           ; p_of_cbm05( 27) = p_terp      ; cbm05_per_megan( 27)  =  1.  
    p_of_megan2cbm05( 28) = is_thujone_a            ; p_of_cbm05( 28) = p_terp      ; cbm05_per_megan( 28)  =  1.  
    p_of_megan2cbm05( 29) = is_thujone_b            ; p_of_cbm05( 29) = p_terp      ; cbm05_per_megan( 29)  =  1.  
    p_of_megan2cbm05( 30) = is_cineole_1_8          ; p_of_cbm05( 30) = p_terp      ; cbm05_per_megan( 30)  =  1.  
    p_of_megan2cbm05( 31) = is_borneol              ; p_of_cbm05( 31) = p_terp      ; cbm05_per_megan( 31)  =  1.  
    p_of_megan2cbm05( 32) = is_linalool             ; p_of_cbm05( 32) = p_terp      ; cbm05_per_megan( 32)  =  1.  
    p_of_megan2cbm05( 33) = is_terpineol_4          ; p_of_cbm05( 33) = p_terp      ; cbm05_per_megan( 33)  =  1.  
    p_of_megan2cbm05( 34) = is_terpineol_a          ; p_of_cbm05( 34) = p_terp      ; cbm05_per_megan( 34)  =  1.  
    p_of_megan2cbm05( 35) = is_linalool_oxd_c       ; p_of_cbm05( 35) = p_terp      ; cbm05_per_megan( 35)  =  1.  
    p_of_megan2cbm05( 36) = is_linalool_oxd_t       ; p_of_cbm05( 36) = p_terp      ; cbm05_per_megan( 36)  =  1.  
    p_of_megan2cbm05( 37) = is_ionone_b             ; p_of_cbm05( 37) = p_isop      ; cbm05_per_megan( 37)  =  3.  
    p_of_megan2cbm05( 38) = is_bornyl_act           ; p_of_cbm05( 38) = p_terp      ; cbm05_per_megan( 38)  =  1.  
    p_of_megan2cbm05( 39) = is_farnescene_a         ; p_of_cbm05( 39) = p_isop      ; cbm05_per_megan( 39)  =  3.  
    p_of_megan2cbm05( 40) = is_caryophyllene_b      ; p_of_cbm05( 40) = p_isop      ; cbm05_per_megan( 40)  =  3.  
    p_of_megan2cbm05( 41) = is_acoradiene           ; p_of_cbm05( 41) = p_isop      ; cbm05_per_megan( 41)  =  3.  
    p_of_megan2cbm05( 42) = is_aromadendrene        ; p_of_cbm05( 42) = p_isop      ; cbm05_per_megan( 42)  =  3.  
    p_of_megan2cbm05( 43) = is_bergamotene_a        ; p_of_cbm05( 43) = p_isop      ; cbm05_per_megan( 43)  =  3.  
    p_of_megan2cbm05( 44) = is_bergamotene_b        ; p_of_cbm05( 44) = p_isop      ; cbm05_per_megan( 44)  =  3.  
    p_of_megan2cbm05( 45) = is_bisabolene_a         ; p_of_cbm05( 45) = p_isop      ; cbm05_per_megan( 45)  =  3.  
    p_of_megan2cbm05( 46) = is_bisabolene_b         ; p_of_cbm05( 46) = p_isop      ; cbm05_per_megan( 46)  =  3.  
    p_of_megan2cbm05( 47) = is_bourbonene_b         ; p_of_cbm05( 47) = p_isop      ; cbm05_per_megan( 47)  =  3.  
    p_of_megan2cbm05( 48) = is_cadinene_d           ; p_of_cbm05( 48) = p_isop      ; cbm05_per_megan( 48)  =  3.  
    p_of_megan2cbm05( 49) = is_cadinene_g           ; p_of_cbm05( 49) = p_isop      ; cbm05_per_megan( 49)  =  3.  
    p_of_megan2cbm05( 50) = is_cedrene_a            ; p_of_cbm05( 50) = p_isop      ; cbm05_per_megan( 50)  =  3.  
    p_of_megan2cbm05( 51) = is_copaene_a            ; p_of_cbm05( 51) = p_isop      ; cbm05_per_megan( 51)  =  3.  
    p_of_megan2cbm05( 52) = is_cubebene_a           ; p_of_cbm05( 52) = p_isop      ; cbm05_per_megan( 52)  =  3.  
    p_of_megan2cbm05( 53) = is_cubebene_b           ; p_of_cbm05( 53) = p_isop      ; cbm05_per_megan( 53)  =  3.  
    p_of_megan2cbm05( 54) = is_elemene_b            ; p_of_cbm05( 54) = p_isop      ; cbm05_per_megan( 54)  =  3.  
    p_of_megan2cbm05( 55) = is_farnescene_b         ; p_of_cbm05( 55) = p_isop      ; cbm05_per_megan( 55)  =  3.  
    p_of_megan2cbm05( 56) = is_germacrene_B         ; p_of_cbm05( 56) = p_isop      ; cbm05_per_megan( 56)  =  3.  
    p_of_megan2cbm05( 57) = is_germacrene_D         ; p_of_cbm05( 57) = p_isop      ; cbm05_per_megan( 57)  =  3.  
    p_of_megan2cbm05( 58) = is_gurjunene_b          ; p_of_cbm05( 58) = p_isop      ; cbm05_per_megan( 58)  =  3.  
    p_of_megan2cbm05( 59) = is_humulene_a           ; p_of_cbm05( 59) = p_isop      ; cbm05_per_megan( 59)  =  3.  
    p_of_megan2cbm05( 60) = is_humulene_g           ; p_of_cbm05( 60) = p_isop      ; cbm05_per_megan( 60)  =  3.  
    p_of_megan2cbm05( 61) = is_isolongifolene       ; p_of_cbm05( 61) = p_isop      ; cbm05_per_megan( 61)  =  3.  
    p_of_megan2cbm05( 62) = is_longifolene          ; p_of_cbm05( 62) = p_isop      ; cbm05_per_megan( 62)  =  3.  
    p_of_megan2cbm05( 63) = is_longipinene          ; p_of_cbm05( 63) = p_isop      ; cbm05_per_megan( 63)  =  3.  
    p_of_megan2cbm05( 64) = is_muurolene_a          ; p_of_cbm05( 64) = p_isop      ; cbm05_per_megan( 64)  =  3.  
    p_of_megan2cbm05( 65) = is_muurolene_g          ; p_of_cbm05( 65) = p_isop      ; cbm05_per_megan( 65)  =  3.  
    p_of_megan2cbm05( 66) = is_selinene_b           ; p_of_cbm05( 66) = p_isop      ; cbm05_per_megan( 66)  =  3.  
    p_of_megan2cbm05( 67) = is_selinene_d           ; p_of_cbm05( 67) = p_isop      ; cbm05_per_megan( 67)  =  3.  
    p_of_megan2cbm05( 68) = is_nerolidol_c          ; p_of_cbm05( 68) = p_isop      ; cbm05_per_megan( 68)  =  3.  
    p_of_megan2cbm05( 69) = is_nerolidol_t          ; p_of_cbm05( 69) = p_isop      ; cbm05_per_megan( 69)  =  3.  
    p_of_megan2cbm05( 70) = is_cedrol               ; p_of_cbm05( 70) = p_isop      ; cbm05_per_megan( 70)  =  3.  
    p_of_megan2cbm05( 71) = is_mbo_2m3e2ol          ; p_of_cbm05( 71) = p_ole      ; cbm05_per_megan( 71)  =  1.  
    p_of_megan2cbm05( 72) = is_mbo_2m3e2ol          ; p_of_cbm05( 72) = p_par      ; cbm05_per_megan( 72)  =  3.  
    p_of_megan2cbm05( 73) = is_methanol             ; p_of_cbm05( 73) = p_meoh     ; cbm05_per_megan( 73)  =  1.  
    p_of_megan2cbm05( 74) = is_acetone              ; p_of_cbm05( 74) = p_par      ; cbm05_per_megan( 74)  =  3.  
    p_of_megan2cbm05( 75) = is_methane              ; p_of_cbm05( 75) = non_react  ; cbm05_per_megan( 75)  =  1   
    p_of_megan2cbm05( 76) = is_ammonia              ; p_of_cbm05( 76) = non_react  ; cbm05_per_megan( 76)  =  1.  
    p_of_megan2cbm05( 77) = is_nitrous_oxd          ; p_of_cbm05( 77) = non_react  ; cbm05_per_megan( 77)  =  1.  
    p_of_megan2cbm05( 78) = is_nitric_oxd           ; p_of_cbm05( 78) = p_no       ; cbm05_per_megan( 78)  =  0. !Soil NO emissions taken from CAMS-GLOB-SOIL (M. Guevara)
    p_of_megan2cbm05( 79) = is_acetaldehyde         ; p_of_cbm05( 79) = p_ald2     ; cbm05_per_megan( 79)  =  1.  
    p_of_megan2cbm05( 80) = is_ethanol              ; p_of_cbm05( 80) = p_etoh     ; cbm05_per_megan( 80)  =  1.  
    p_of_megan2cbm05( 81) = is_formic_acid          ; p_of_cbm05( 81) = non_react  ; cbm05_per_megan( 81)  =  1.  
    p_of_megan2cbm05( 82) = is_formaldehyde         ; p_of_cbm05( 82) = p_form     ; cbm05_per_megan( 82)  =  1.  
    p_of_megan2cbm05( 83) = is_acetic_acid          ; p_of_cbm05( 83) = p_par      ; cbm05_per_megan( 83)  =  1.  
    p_of_megan2cbm05( 84) = is_mbo_3m2e1ol          ; p_of_cbm05( 84) = p_aldx     ; cbm05_per_megan( 84)  =  1.  
    p_of_megan2cbm05( 85) = is_mbo_3m2e1ol          ; p_of_cbm05( 85) = p_par      ; cbm05_per_megan( 85)  =  3.  
    p_of_megan2cbm05( 86) = is_mbo_3m3e1ol          ; p_of_cbm05( 86) = p_form     ; cbm05_per_megan( 86)  =  1.  
    p_of_megan2cbm05( 87) = is_mbo_3m3e1ol          ; p_of_cbm05( 87) = p_par      ; cbm05_per_megan( 87)  =  4.  
    p_of_megan2cbm05( 88) = is_benzaldehyde         ; p_of_cbm05( 88) = non_react  ; cbm05_per_megan( 88)  =  1.  
    p_of_megan2cbm05( 89) = is_butanone_2           ; p_of_cbm05( 89) = p_par      ; cbm05_per_megan( 89)  =  4.  
    p_of_megan2cbm05( 90) = is_butanone_2           ; p_of_cbm05( 90) = p_par      ; cbm05_per_megan( 90)  =  0.  
    p_of_megan2cbm05( 91) = is_decanal              ; p_of_cbm05( 91) = p_aldx     ; cbm05_per_megan( 91)  =  1.  
    p_of_megan2cbm05( 92) = is_decanal              ; p_of_cbm05( 92) = p_par      ; cbm05_per_megan( 92)  =  8.  
    p_of_megan2cbm05( 93) = is_dodecene_1           ; p_of_cbm05( 93) = p_ole      ; cbm05_per_megan( 93)  =  1.  
    p_of_megan2cbm05( 94) = is_dodecene_1           ; p_of_cbm05( 94) = p_par      ; cbm05_per_megan( 94)  =  10. 
    p_of_megan2cbm05( 95) = is_geranyl_acetone      ; p_of_cbm05( 95) = p_isop      ; cbm05_per_megan( 95)  =  3.  
    p_of_megan2cbm05( 96) = is_heptanal             ; p_of_cbm05( 96) = p_aldx     ; cbm05_per_megan( 96)  =  1.  
    p_of_megan2cbm05( 97) = is_heptanal             ; p_of_cbm05( 97) = p_par      ; cbm05_per_megan( 97)  =  5.  
    p_of_megan2cbm05( 98) = is_heptane              ; p_of_cbm05( 98) = p_par      ; cbm05_per_megan( 98)  =  7.  
    p_of_megan2cbm05( 99) = is_hexane               ; p_of_cbm05( 99) = p_par      ; cbm05_per_megan( 99)  =  6.  
    p_of_megan2cbm05(100) = is_met_benzoate         ; p_of_cbm05(100) = p_tol      ; cbm05_per_megan(100)  =  1.  
    p_of_megan2cbm05(101) = is_met_heptenone        ; p_of_cbm05(101) = non_react      ; cbm05_per_megan(101)  =  0.  
    p_of_megan2cbm05(102) = is_met_heptenone        ; p_of_cbm05(102) = p_par      ; cbm05_per_megan(102)  =  5.  
    p_of_megan2cbm05(103) = is_met_heptenone        ; p_of_cbm05(103) = p_ole      ; cbm05_per_megan(103)  =  1.  
    p_of_megan2cbm05(104) = is_neryl_acetone        ; p_of_cbm05(104) = non_react      ; cbm05_per_megan(104)  =  0.  
    p_of_megan2cbm05(105) = is_neryl_acetone        ; p_of_cbm05(105) = p_par      ; cbm05_per_megan(105)  = 10.  
    p_of_megan2cbm05(106) = is_neryl_acetone        ; p_of_cbm05(106) = p_iole      ; cbm05_per_megan(106)  =  2.  
    p_of_megan2cbm05(107) = is_nonanal              ; p_of_cbm05(107) = p_aldx     ; cbm05_per_megan(107)  =  1.  
    p_of_megan2cbm05(108) = is_nonanal              ; p_of_cbm05(108) = p_par      ; cbm05_per_megan(108)  =  7.  
    p_of_megan2cbm05(109) = is_nonenal              ; p_of_cbm05(109) = p_aldx     ; cbm05_per_megan(109)  =  1.  
    p_of_megan2cbm05(110) = is_nonenal              ; p_of_cbm05(110) = p_par      ; cbm05_per_megan(110)  =  6.  
    p_of_megan2cbm05(111) = is_nonenal              ; p_of_cbm05(111) = p_iole      ; cbm05_per_megan(111)  =  1.  
    p_of_megan2cbm05(112) = is_octanal              ; p_of_cbm05(112) = p_aldx     ; cbm05_per_megan(112)  =  1.  
    p_of_megan2cbm05(113) = is_octanal              ; p_of_cbm05(113) = p_par      ; cbm05_per_megan(113)  =  6.  
    p_of_megan2cbm05(114) = is_octanol              ; p_of_cbm05(114) = p_par      ; cbm05_per_megan(114)  =  8.  
    p_of_megan2cbm05(115) = is_octenol_1e3ol        ; p_of_cbm05(115) = p_par      ; cbm05_per_megan(115)  =  6.  
    p_of_megan2cbm05(116) = is_octenol_1e3ol        ; p_of_cbm05(116) = p_ole      ; cbm05_per_megan(116)  =  1.  
    p_of_megan2cbm05(117) = is_oxopentanal          ; p_of_cbm05(117) = p_aldx     ; cbm05_per_megan(117)  =  1.  
    p_of_megan2cbm05(118) = is_oxopentanal          ; p_of_cbm05(118) = p_par      ; cbm05_per_megan(118)  =  3.  
    p_of_megan2cbm05(119) = is_pentane              ; p_of_cbm05(119) = p_par      ; cbm05_per_megan(119)  =  5.  
    p_of_megan2cbm05(120) = is_phenyl_cco           ; p_of_cbm05(120) = p_aldx     ; cbm05_per_megan(120)  =  1   
    p_of_megan2cbm05(121) = is_phenyl_cco           ; p_of_cbm05(121) = p_tol      ; cbm05_per_megan(121)  =  1.  
    p_of_megan2cbm05(122) = is_pyruvic_acid         ; p_of_cbm05(122) = non_react  ; cbm05_per_megan(122)  =  1.  
    p_of_megan2cbm05(123) = is_pyruvic_acid         ; p_of_cbm05(123) = non_react  ; cbm05_per_megan(123)  =  1.  
    p_of_megan2cbm05(124) = is_terpinyl_act_a       ; p_of_cbm05(124) = p_isop     ; cbm05_per_megan(124)  =  2.  
    p_of_megan2cbm05(125) = is_tetradecene_1        ; p_of_cbm05(125) = p_par      ; cbm05_per_megan(125)  =  12. 
    p_of_megan2cbm05(126) = is_tetradecene_1        ; p_of_cbm05(126) = p_ole      ; cbm05_per_megan(126)  =  1.  
    p_of_megan2cbm05(127) = is_toluene              ; p_of_cbm05(127) = p_tol      ; cbm05_per_megan(127)  =  1.  
    p_of_megan2cbm05(128) = is_carbon_monoxide      ; p_of_cbm05(128) = p_co       ; cbm05_per_megan(128)  =  1.  
    p_of_megan2cbm05(129) = is_butene               ; p_of_cbm05(129) = p_ole      ; cbm05_per_megan(129)  =  1.  
    p_of_megan2cbm05(130) = is_butene               ; p_of_cbm05(130) = p_par      ; cbm05_per_megan(130)  =  2.  
    p_of_megan2cbm05(131) = is_ethane               ; p_of_cbm05(131) = p_etha     ; cbm05_per_megan(131)  =  1.  
    p_of_megan2cbm05(132) = is_ethene               ; p_of_cbm05(132) = p_eth      ; cbm05_per_megan(132)  =  1.  
    p_of_megan2cbm05(133) = is_hydrogen_cyanide     ; p_of_cbm05(133) = non_react  ; cbm05_per_megan(133)  =  1.  
    p_of_megan2cbm05(134) = is_propane              ; p_of_cbm05(134) = p_par      ; cbm05_per_megan(134)  =  1.5  
    p_of_megan2cbm05(135) = is_propene              ; p_of_cbm05(135) = p_ole      ; cbm05_per_megan(135)  =  1.  
    p_of_megan2cbm05(136) = is_propene              ; p_of_cbm05(136) = p_par      ; cbm05_per_megan(136)  =  1.  
    p_of_megan2cbm05(137) = is_carbon_2s            ; p_of_cbm05(137) = non_react  ; cbm05_per_megan(137)  =  1.  
    p_of_megan2cbm05(138) = is_carbonyl_s           ; p_of_cbm05(138) = non_react  ; cbm05_per_megan(138)  =  1.  
    p_of_megan2cbm05(139) = is_diallyl_2s           ; p_of_cbm05(139) = non_react  ; cbm05_per_megan(139)  =  1.  
    p_of_megan2cbm05(140) = is_diallyl_2s           ; p_of_cbm05(140) = p_par      ; cbm05_per_megan(140)  =  4.  
    p_of_megan2cbm05(141) = is_diallyl_2s           ; p_of_cbm05(141) = p_ole      ; cbm05_per_megan(141)  =  2.  
    p_of_megan2cbm05(142) = is_2met_2s              ; p_of_cbm05(142) = p_par      ; cbm05_per_megan(142)  =  2.
    p_of_megan2cbm05(143) = is_2met_s               ; p_of_cbm05(143) = p_par      ; cbm05_per_megan(143)  =  2.  
    p_of_megan2cbm05(144) = is_met_chloride         ; p_of_cbm05(144) = non_react  ; cbm05_per_megan(144)  =  1.  
    p_of_megan2cbm05(145) = is_met_bromide          ; p_of_cbm05(145) = non_react  ; cbm05_per_megan(145)  =  1.  
    p_of_megan2cbm05(146) = is_met_iodide           ; p_of_cbm05(146) = non_react  ; cbm05_per_megan(146)  =  1.  
    p_of_megan2cbm05(147) = is_hydrogen_s           ; p_of_cbm05(147) = non_react  ; cbm05_per_megan(147)  =  1.  
    p_of_megan2cbm05(148) = is_met_mercaptan        ; p_of_cbm05(148) = p_par      ; cbm05_per_megan(148)  =  1.  
    p_of_megan2cbm05(149) = is_met_propenyl_2s      ; p_of_cbm05(149) = p_par      ; cbm05_per_megan(149)  =  2.
    p_of_megan2cbm05(150) = is_met_propenyl_2s      ; p_of_cbm05(150) = p_iole      ; cbm05_per_megan(150)  =  1.  
    p_of_megan2cbm05(151) = is_pppp_2s              ; p_of_cbm05(151) = p_par      ; cbm05_per_megan(151)  =  2.  
    p_of_megan2cbm05(152) = is_pppp_2s              ; p_of_cbm05(152) = p_par      ; cbm05_per_megan(152)  =  2.  
    p_of_megan2cbm05(153) = is_pppp_2s              ; p_of_cbm05(153) = p_iole      ; cbm05_per_megan(153)  =  1.  
    p_of_megan2cbm05(154) = is_2met_nonatriene      ; p_of_cbm05(154) = p_terp      ; cbm05_per_megan(154)  =  1.  
    p_of_megan2cbm05(155) = is_met_salicylate       ; p_of_cbm05(155) = p_tol      ; cbm05_per_megan(155)  =  1.  
    p_of_megan2cbm05(156) = is_indole               ; p_of_cbm05(156) = p_tol      ; cbm05_per_megan(156)  =  1.  
    p_of_megan2cbm05(157) = is_jasmone              ; p_of_cbm05(157) = p_isop      ; cbm05_per_megan(157)  =  2.  
    p_of_megan2cbm05(158) = is_met_jasmonate        ; p_of_cbm05(158) = p_isop      ; cbm05_per_megan(158)  =  3.  
    p_of_megan2cbm05(159) = is_3met_3dctt           ; p_of_cbm05(159) = p_isop      ; cbm05_per_megan(159)  =  3.  
    p_of_megan2cbm05(160) = is_hexanal              ; p_of_cbm05(160) = p_aldx     ; cbm05_per_megan(160)  =  1.  
    p_of_megan2cbm05(161) = is_hexanal              ; p_of_cbm05(161) = p_par      ; cbm05_per_megan(161)  =  4.  
    p_of_megan2cbm05(162) = is_hexanol_1            ; p_of_cbm05(162) = p_par      ; cbm05_per_megan(162)  =  6.  
    p_of_megan2cbm05(163) = is_hexenal_c3           ; p_of_cbm05(163) = p_aldx     ; cbm05_per_megan(163)  =  1.  
    p_of_megan2cbm05(164) = is_hexenal_c3           ; p_of_cbm05(164) = p_par      ; cbm05_per_megan(164)  =  3.  
    p_of_megan2cbm05(165) = is_hexenal_c3           ; p_of_cbm05(165) = p_iole      ; cbm05_per_megan(165)  =  1   
    p_of_megan2cbm05(166) = is_hexenal_t2           ; p_of_cbm05(166) = p_aldx     ; cbm05_per_megan(166)  =  1.  
    p_of_megan2cbm05(167) = is_hexenal_t2           ; p_of_cbm05(167) = p_par      ; cbm05_per_megan(167)  =  6.  
    p_of_megan2cbm05(168) = is_hexenal_t2           ; p_of_cbm05(168) = p_iole      ; cbm05_per_megan(168)  =  1.  
    p_of_megan2cbm05(169) = is_hexenol_c3           ; p_of_cbm05(169) = p_par      ; cbm05_per_megan(169)  =  5.  
    p_of_megan2cbm05(170) = is_hexenol_c3           ; p_of_cbm05(170) = p_iole      ; cbm05_per_megan(170)  =  1.  
    p_of_megan2cbm05(171) = is_hexenyl_act_c3       ; p_of_cbm05(171) = non_react  ; cbm05_per_megan(171)  =  1.  
    p_of_megan2cbm05(172) = is_hexenyl_act_c3       ; p_of_cbm05(172) = p_par      ; cbm05_per_megan(172)  =  5.  
    p_of_megan2cbm05(173) = is_hexenyl_act_c3       ; p_of_cbm05(173) = p_iole      ; cbm05_per_megan(173)  =  1.  

  END SUBROUTINE get_megan2cbm05_table

  !--------------------------------------------------------------------
  SUBROUTINE get_megan2sulsoa_global_table


    ! For MEGAN v2.04 species conversion to bsc sulsoa_global species 

    !
    ! Index of                                       Index of                      Molar ratio
    ! MEGAN species                                  CMB05 Species                             
    !
    p_of_megan2cbm05(  1) = is_isoprene             ; p_of_cbm05(  1) = p_isop      ; cbm05_per_megan(  1)  =  1.
    p_of_megan2cbm05(  2) = is_myrcene              ; p_of_cbm05(  2) = p_pinene_a  ; cbm05_per_megan(  2)  =  1.
    p_of_megan2cbm05(  3) = is_sabinene             ; p_of_cbm05(  3) = p_pinene_a  ; cbm05_per_megan(  3)  =  1.
    p_of_megan2cbm05(  4) = is_limonene             ; p_of_cbm05(  4) = p_pinene_a  ; cbm05_per_megan(  4)  =  1.
    p_of_megan2cbm05(  5) = is_carene_3             ; p_of_cbm05(  5) = p_pinene_a  ; cbm05_per_megan(  5)  =  1.
    p_of_megan2cbm05(  6) = is_ocimene_t_b          ; p_of_cbm05(  6) = p_pinene_a  ; cbm05_per_megan(  6)  =  1.
    p_of_megan2cbm05(  7) = is_pinene_b             ; p_of_cbm05(  7) = p_pinene_a  ; cbm05_per_megan(  7)  =  1.
    p_of_megan2cbm05(  8) = is_pinene_a             ; p_of_cbm05(  8) = p_pinene_a  ; cbm05_per_megan(  8)  =  1.
    p_of_megan2cbm05(  9) = is_2met_styrene         ; p_of_cbm05(  9) = p_pinene_a  ; cbm05_per_megan(  9)  =  1.
    p_of_megan2cbm05( 10) = is_cymene_p             ; p_of_cbm05( 10) = p_pinene_a  ; cbm05_per_megan( 10)  =  1.
    p_of_megan2cbm05( 11) = is_cymene_o             ; p_of_cbm05( 11) = p_pinene_a  ; cbm05_per_megan( 11)  =  1.
    p_of_megan2cbm05( 12) = is_phellandrene_a       ; p_of_cbm05( 12) = p_pinene_a  ; cbm05_per_megan( 12)  =  1.
    p_of_megan2cbm05( 13) = is_thujene_a            ; p_of_cbm05( 13) = p_pinene_a  ; cbm05_per_megan( 13)  =  1.
    p_of_megan2cbm05( 14) = is_terpinene_a          ; p_of_cbm05( 14) = p_pinene_a  ; cbm05_per_megan( 14)  =  1.
    p_of_megan2cbm05( 15) = is_terpinene_g          ; p_of_cbm05( 15) = p_pinene_a  ; cbm05_per_megan( 15)  =  1.
    p_of_megan2cbm05( 16) = is_terpinolene          ; p_of_cbm05( 16) = p_pinene_a  ; cbm05_per_megan( 16)  =  1.
    p_of_megan2cbm05( 17) = is_phellandrene_b       ; p_of_cbm05( 17) = p_pinene_a  ; cbm05_per_megan( 17)  =  1.
    p_of_megan2cbm05( 18) = is_camphene             ; p_of_cbm05( 18) = p_pinene_a  ; cbm05_per_megan( 18)  =  1.
    p_of_megan2cbm05( 19) = is_bornene              ; p_of_cbm05( 19) = p_pinene_a  ; cbm05_per_megan( 19)  =  1.
    p_of_megan2cbm05( 20) = is_fenchene_a           ; p_of_cbm05( 20) = p_pinene_a  ; cbm05_per_megan( 20)  =  1.
    p_of_megan2cbm05( 21) = is_ocimene_al           ; p_of_cbm05( 21) = p_pinene_a  ; cbm05_per_megan( 21)  =  1.
    p_of_megan2cbm05( 22) = is_ocimene_c_b          ; p_of_cbm05( 22) = p_pinene_a  ; cbm05_per_megan( 22)  =  1.
    p_of_megan2cbm05( 23) = is_tricyclene           ; p_of_cbm05( 23) = non_react   ; cbm05_per_megan( 23)  =  1.  
    p_of_megan2cbm05( 24) = is_estragole            ; p_of_cbm05( 24) = non_react   ; cbm05_per_megan( 24)  =  1.  
    p_of_megan2cbm05( 25) = is_camphor              ; p_of_cbm05( 25) = non_react   ; cbm05_per_megan( 25)  =  1.  
    p_of_megan2cbm05( 26) = is_fenchone             ; p_of_cbm05( 26) = non_react   ; cbm05_per_megan( 26)  =  1.  
    p_of_megan2cbm05( 27) = is_piperitone           ; p_of_cbm05( 27) = non_react   ; cbm05_per_megan( 27)  =  1.  
    p_of_megan2cbm05( 28) = is_thujone_a            ; p_of_cbm05( 28) = non_react   ; cbm05_per_megan( 28)  =  1.  
    p_of_megan2cbm05( 29) = is_thujone_b            ; p_of_cbm05( 29) = non_react   ; cbm05_per_megan( 29)  =  1.  
    p_of_megan2cbm05( 30) = is_cineole_1_8          ; p_of_cbm05( 30) = non_react   ; cbm05_per_megan( 30)  =  1.  
    p_of_megan2cbm05( 31) = is_borneol              ; p_of_cbm05( 31) = non_react   ; cbm05_per_megan( 31)  =  1.  
    p_of_megan2cbm05( 32) = is_linalool             ; p_of_cbm05( 32) = non_react   ; cbm05_per_megan( 32)  =  1.  
    p_of_megan2cbm05( 33) = is_terpineol_4          ; p_of_cbm05( 33) = non_react   ; cbm05_per_megan( 33)  =  1.  
    p_of_megan2cbm05( 34) = is_terpineol_a          ; p_of_cbm05( 34) = non_react   ; cbm05_per_megan( 34)  =  1.  
    p_of_megan2cbm05( 35) = is_linalool_oxd_c       ; p_of_cbm05( 35) = non_react   ; cbm05_per_megan( 35)  =  1.  
    p_of_megan2cbm05( 36) = is_linalool_oxd_t       ; p_of_cbm05( 36) = non_react   ; cbm05_per_megan( 36)  =  1.  
    p_of_megan2cbm05( 37) = is_ionone_b             ; p_of_cbm05( 37) = non_react   ; cbm05_per_megan( 37)  =  3.  
    p_of_megan2cbm05( 38) = is_bornyl_act           ; p_of_cbm05( 38) = non_react   ; cbm05_per_megan( 38)  =  1.  
    p_of_megan2cbm05( 39) = is_farnescene_a         ; p_of_cbm05( 39) = non_react   ; cbm05_per_megan( 39)  =  3.  
    p_of_megan2cbm05( 40) = is_caryophyllene_b      ; p_of_cbm05( 40) = non_react   ; cbm05_per_megan( 40)  =  3.  
    p_of_megan2cbm05( 41) = is_acoradiene           ; p_of_cbm05( 41) = non_react   ; cbm05_per_megan( 41)  =  3.  
    p_of_megan2cbm05( 42) = is_aromadendrene        ; p_of_cbm05( 42) = non_react   ; cbm05_per_megan( 42)  =  3.  
    p_of_megan2cbm05( 43) = is_bergamotene_a        ; p_of_cbm05( 43) = non_react   ; cbm05_per_megan( 43)  =  3.  
    p_of_megan2cbm05( 44) = is_bergamotene_b        ; p_of_cbm05( 44) = non_react   ; cbm05_per_megan( 44)  =  3.  
    p_of_megan2cbm05( 45) = is_bisabolene_a         ; p_of_cbm05( 45) = non_react   ; cbm05_per_megan( 45)  =  3.  
    p_of_megan2cbm05( 46) = is_bisabolene_b         ; p_of_cbm05( 46) = non_react   ; cbm05_per_megan( 46)  =  3.  
    p_of_megan2cbm05( 47) = is_bourbonene_b         ; p_of_cbm05( 47) = non_react   ; cbm05_per_megan( 47)  =  3.  
    p_of_megan2cbm05( 48) = is_cadinene_d           ; p_of_cbm05( 48) = non_react   ; cbm05_per_megan( 48)  =  3.  
    p_of_megan2cbm05( 49) = is_cadinene_g           ; p_of_cbm05( 49) = non_react   ; cbm05_per_megan( 49)  =  3.  
    p_of_megan2cbm05( 50) = is_cedrene_a            ; p_of_cbm05( 50) = non_react   ; cbm05_per_megan( 50)  =  3.  
    p_of_megan2cbm05( 51) = is_copaene_a            ; p_of_cbm05( 51) = non_react   ; cbm05_per_megan( 51)  =  3.  
    p_of_megan2cbm05( 52) = is_cubebene_a           ; p_of_cbm05( 52) = non_react   ; cbm05_per_megan( 52)  =  3.  
    p_of_megan2cbm05( 53) = is_cubebene_b           ; p_of_cbm05( 53) = non_react   ; cbm05_per_megan( 53)  =  3.  
    p_of_megan2cbm05( 54) = is_elemene_b            ; p_of_cbm05( 54) = non_react   ; cbm05_per_megan( 54)  =  3.  
    p_of_megan2cbm05( 55) = is_farnescene_b         ; p_of_cbm05( 55) = non_react   ; cbm05_per_megan( 55)  =  3.  
    p_of_megan2cbm05( 56) = is_germacrene_B         ; p_of_cbm05( 56) = non_react   ; cbm05_per_megan( 56)  =  3.  
    p_of_megan2cbm05( 57) = is_germacrene_D         ; p_of_cbm05( 57) = non_react   ; cbm05_per_megan( 57)  =  3.  
    p_of_megan2cbm05( 58) = is_gurjunene_b          ; p_of_cbm05( 58) = non_react   ; cbm05_per_megan( 58)  =  3.  
    p_of_megan2cbm05( 59) = is_humulene_a           ; p_of_cbm05( 59) = non_react   ; cbm05_per_megan( 59)  =  3.  
    p_of_megan2cbm05( 60) = is_humulene_g           ; p_of_cbm05( 60) = non_react   ; cbm05_per_megan( 60)  =  3.  
    p_of_megan2cbm05( 61) = is_isolongifolene       ; p_of_cbm05( 61) = non_react   ; cbm05_per_megan( 61)  =  3.  
    p_of_megan2cbm05( 62) = is_longifolene          ; p_of_cbm05( 62) = non_react   ; cbm05_per_megan( 62)  =  3.  
    p_of_megan2cbm05( 63) = is_longipinene          ; p_of_cbm05( 63) = non_react   ; cbm05_per_megan( 63)  =  3.  
    p_of_megan2cbm05( 64) = is_muurolene_a          ; p_of_cbm05( 64) = non_react   ; cbm05_per_megan( 64)  =  3.  
    p_of_megan2cbm05( 65) = is_muurolene_g          ; p_of_cbm05( 65) = non_react   ; cbm05_per_megan( 65)  =  3.  
    p_of_megan2cbm05( 66) = is_selinene_b           ; p_of_cbm05( 66) = non_react   ; cbm05_per_megan( 66)  =  3.  
    p_of_megan2cbm05( 67) = is_selinene_d           ; p_of_cbm05( 67) = non_react   ; cbm05_per_megan( 67)  =  3.  
    p_of_megan2cbm05( 68) = is_nerolidol_c          ; p_of_cbm05( 68) = non_react   ; cbm05_per_megan( 68)  =  3.  
    p_of_megan2cbm05( 69) = is_nerolidol_t          ; p_of_cbm05( 69) = non_react   ; cbm05_per_megan( 69)  =  3.  
    p_of_megan2cbm05( 70) = is_cedrol               ; p_of_cbm05( 70) = non_react   ; cbm05_per_megan( 70)  =  3.  
    p_of_megan2cbm05( 71) = is_mbo_2m3e2ol          ; p_of_cbm05( 71) = non_react  ; cbm05_per_megan( 71)  =  1.  
    p_of_megan2cbm05( 72) = is_mbo_2m3e2ol          ; p_of_cbm05( 72) = non_react  ; cbm05_per_megan( 72)  =  3.  
    p_of_megan2cbm05( 73) = is_methanol             ; p_of_cbm05( 73) = non_react  ; cbm05_per_megan( 73)  =  1.  
    p_of_megan2cbm05( 74) = is_acetone              ; p_of_cbm05( 74) = non_react  ; cbm05_per_megan( 74)  =  3.  
    p_of_megan2cbm05( 75) = is_methane              ; p_of_cbm05( 75) = non_react  ; cbm05_per_megan( 75)  =  1   
    p_of_megan2cbm05( 76) = is_ammonia              ; p_of_cbm05( 76) = non_react  ; cbm05_per_megan( 76)  =  1.  
    p_of_megan2cbm05( 77) = is_nitrous_oxd          ; p_of_cbm05( 77) = non_react  ; cbm05_per_megan( 77)  =  1.  
    p_of_megan2cbm05( 78) = is_nitric_oxd           ; p_of_cbm05( 78) = non_react  ; cbm05_per_megan( 78)  =  1.  
    p_of_megan2cbm05( 79) = is_acetaldehyde         ; p_of_cbm05( 79) = non_react  ; cbm05_per_megan( 79)  =  1.  
    p_of_megan2cbm05( 80) = is_ethanol              ; p_of_cbm05( 80) = non_react  ; cbm05_per_megan( 80)  =  1.  
    p_of_megan2cbm05( 81) = is_formic_acid          ; p_of_cbm05( 81) = non_react  ; cbm05_per_megan( 81)  =  1.  
    p_of_megan2cbm05( 82) = is_formaldehyde         ; p_of_cbm05( 82) = non_react  ; cbm05_per_megan( 82)  =  1.  
    p_of_megan2cbm05( 83) = is_acetic_acid          ; p_of_cbm05( 83) = non_react  ; cbm05_per_megan( 83)  =  1.  
    p_of_megan2cbm05( 84) = is_mbo_3m2e1ol          ; p_of_cbm05( 84) = non_react  ; cbm05_per_megan( 84)  =  1.  
    p_of_megan2cbm05( 85) = is_mbo_3m2e1ol          ; p_of_cbm05( 85) = non_react  ; cbm05_per_megan( 85)  =  3.  
    p_of_megan2cbm05( 86) = is_mbo_3m3e1ol          ; p_of_cbm05( 86) = non_react  ; cbm05_per_megan( 86)  =  1.  
    p_of_megan2cbm05( 87) = is_mbo_3m3e1ol          ; p_of_cbm05( 87) = non_react  ; cbm05_per_megan( 87)  =  4.  
    p_of_megan2cbm05( 88) = is_benzaldehyde         ; p_of_cbm05( 88) = non_react  ; cbm05_per_megan( 88)  =  1.  
    p_of_megan2cbm05( 89) = is_butanone_2           ; p_of_cbm05( 89) = non_react  ; cbm05_per_megan( 89)  =  4.  
    p_of_megan2cbm05( 90) = is_butanone_2           ; p_of_cbm05( 90) = non_react  ; cbm05_per_megan( 90)  =  0.  
    p_of_megan2cbm05( 91) = is_decanal              ; p_of_cbm05( 91) = non_react  ; cbm05_per_megan( 91)  =  1.  
    p_of_megan2cbm05( 92) = is_decanal              ; p_of_cbm05( 92) = non_react  ; cbm05_per_megan( 92)  =  8.  
    p_of_megan2cbm05( 93) = is_dodecene_1           ; p_of_cbm05( 93) = non_react  ; cbm05_per_megan( 93)  =  1.  
    p_of_megan2cbm05( 94) = is_dodecene_1           ; p_of_cbm05( 94) = non_react  ; cbm05_per_megan( 94)  =  10. 
    p_of_megan2cbm05( 95) = is_geranyl_acetone      ; p_of_cbm05( 95) = non_react   ; cbm05_per_megan( 95)  =  3.  
    p_of_megan2cbm05( 96) = is_heptanal             ; p_of_cbm05( 96) = non_react  ; cbm05_per_megan( 96)  =  1.  
    p_of_megan2cbm05( 97) = is_heptanal             ; p_of_cbm05( 97) = non_react  ; cbm05_per_megan( 97)  =  5.  
    p_of_megan2cbm05( 98) = is_heptane              ; p_of_cbm05( 98) = non_react  ; cbm05_per_megan( 98)  =  7.  
    p_of_megan2cbm05( 99) = is_hexane               ; p_of_cbm05( 99) = non_react  ; cbm05_per_megan( 99)  =  6.  
    p_of_megan2cbm05(100) = is_met_benzoate         ; p_of_cbm05(100) = non_react  ; cbm05_per_megan(100)  =  1.  
    p_of_megan2cbm05(101) = is_met_heptenone        ; p_of_cbm05(101) = non_react      ; cbm05_per_megan(101)  =  0.  
    p_of_megan2cbm05(102) = is_met_heptenone        ; p_of_cbm05(102) = non_react  ; cbm05_per_megan(102)  =  5.  
    p_of_megan2cbm05(103) = is_met_heptenone        ; p_of_cbm05(103) = non_react  ; cbm05_per_megan(103)  =  1.  
    p_of_megan2cbm05(104) = is_neryl_acetone        ; p_of_cbm05(104) = non_react      ; cbm05_per_megan(104)  =  0.  
    p_of_megan2cbm05(105) = is_neryl_acetone        ; p_of_cbm05(105) = non_react  ; cbm05_per_megan(105)  = 10.  
    p_of_megan2cbm05(106) = is_neryl_acetone        ; p_of_cbm05(106) = non_react   ; cbm05_per_megan(106)  =  2.  
    p_of_megan2cbm05(107) = is_nonanal              ; p_of_cbm05(107) = non_react  ; cbm05_per_megan(107)  =  1.  
    p_of_megan2cbm05(108) = is_nonanal              ; p_of_cbm05(108) = non_react  ; cbm05_per_megan(108)  =  7.  
    p_of_megan2cbm05(109) = is_nonenal              ; p_of_cbm05(109) = non_react  ; cbm05_per_megan(109)  =  1.  
    p_of_megan2cbm05(110) = is_nonenal              ; p_of_cbm05(110) = non_react  ; cbm05_per_megan(110)  =  6.  
    p_of_megan2cbm05(111) = is_nonenal              ; p_of_cbm05(111) = non_react   ; cbm05_per_megan(111)  =  1.  
    p_of_megan2cbm05(112) = is_octanal              ; p_of_cbm05(112) = non_react  ; cbm05_per_megan(112)  =  1.  
    p_of_megan2cbm05(113) = is_octanal              ; p_of_cbm05(113) = non_react  ; cbm05_per_megan(113)  =  6.  
    p_of_megan2cbm05(114) = is_octanol              ; p_of_cbm05(114) = non_react  ; cbm05_per_megan(114)  =  8.  
    p_of_megan2cbm05(115) = is_octenol_1e3ol        ; p_of_cbm05(115) = non_react  ; cbm05_per_megan(115)  =  6.  
    p_of_megan2cbm05(116) = is_octenol_1e3ol        ; p_of_cbm05(116) = non_react  ; cbm05_per_megan(116)  =  1.  
    p_of_megan2cbm05(117) = is_oxopentanal          ; p_of_cbm05(117) = non_react  ; cbm05_per_megan(117)  =  1.  
    p_of_megan2cbm05(118) = is_oxopentanal          ; p_of_cbm05(118) = non_react  ; cbm05_per_megan(118)  =  3.  
    p_of_megan2cbm05(119) = is_pentane              ; p_of_cbm05(119) = non_react  ; cbm05_per_megan(119)  =  5.  
    p_of_megan2cbm05(120) = is_phenyl_cco           ; p_of_cbm05(120) = non_react  ; cbm05_per_megan(120)  =  1   
    p_of_megan2cbm05(121) = is_phenyl_cco           ; p_of_cbm05(121) = non_react  ; cbm05_per_megan(121)  =  1.  
    p_of_megan2cbm05(122) = is_pyruvic_acid         ; p_of_cbm05(122) = non_react  ; cbm05_per_megan(122)  =  1.  
    p_of_megan2cbm05(123) = is_pyruvic_acid         ; p_of_cbm05(123) = non_react  ; cbm05_per_megan(123)  =  1.  
    p_of_megan2cbm05(124) = is_terpinyl_act_a       ; p_of_cbm05(124) = non_react  ; cbm05_per_megan(124)  =  2.  
    p_of_megan2cbm05(125) = is_tetradecene_1        ; p_of_cbm05(125) = non_react  ; cbm05_per_megan(125)  =  12. 
    p_of_megan2cbm05(126) = is_tetradecene_1        ; p_of_cbm05(126) = non_react  ; cbm05_per_megan(126)  =  1.  
    p_of_megan2cbm05(127) = is_toluene              ; p_of_cbm05(127) = p_tol      ; cbm05_per_megan(127)  =  1.  
    p_of_megan2cbm05(128) = is_carbon_monoxide      ; p_of_cbm05(128) = non_react  ; cbm05_per_megan(128)  =  1.  
    p_of_megan2cbm05(129) = is_butene               ; p_of_cbm05(129) = non_react  ; cbm05_per_megan(129)  =  1.  
    p_of_megan2cbm05(130) = is_butene               ; p_of_cbm05(130) = non_react  ; cbm05_per_megan(130)  =  2.  
    p_of_megan2cbm05(131) = is_ethane               ; p_of_cbm05(131) = non_react  ; cbm05_per_megan(131)  =  1.  
    p_of_megan2cbm05(132) = is_ethene               ; p_of_cbm05(132) = non_react  ; cbm05_per_megan(132)  =  1.  
    p_of_megan2cbm05(133) = is_hydrogen_cyanide     ; p_of_cbm05(133) = non_react  ; cbm05_per_megan(133)  =  1.  
    p_of_megan2cbm05(134) = is_propane              ; p_of_cbm05(134) = non_react  ; cbm05_per_megan(134)  =  1.5  
    p_of_megan2cbm05(135) = is_propene              ; p_of_cbm05(135) = non_react  ; cbm05_per_megan(135)  =  1.  
    p_of_megan2cbm05(136) = is_propene              ; p_of_cbm05(136) = non_react  ; cbm05_per_megan(136)  =  1.  
    p_of_megan2cbm05(137) = is_carbon_2s            ; p_of_cbm05(137) = non_react  ; cbm05_per_megan(137)  =  1.  
    p_of_megan2cbm05(138) = is_carbonyl_s           ; p_of_cbm05(138) = non_react  ; cbm05_per_megan(138)  =  1.  
    p_of_megan2cbm05(139) = is_diallyl_2s           ; p_of_cbm05(139) = non_react  ; cbm05_per_megan(139)  =  1.  
    p_of_megan2cbm05(140) = is_diallyl_2s           ; p_of_cbm05(140) = non_react  ; cbm05_per_megan(140)  =  4.  
    p_of_megan2cbm05(141) = is_diallyl_2s           ; p_of_cbm05(141) = non_react  ; cbm05_per_megan(141)  =  2.  
    p_of_megan2cbm05(142) = is_2met_2s              ; p_of_cbm05(142) = non_react  ; cbm05_per_megan(142)  =  2.
    p_of_megan2cbm05(143) = is_2met_s               ; p_of_cbm05(143) = non_react  ; cbm05_per_megan(143)  =  2.  
    p_of_megan2cbm05(144) = is_met_chloride         ; p_of_cbm05(144) = non_react  ; cbm05_per_megan(144)  =  1.  
    p_of_megan2cbm05(145) = is_met_bromide          ; p_of_cbm05(145) = non_react  ; cbm05_per_megan(145)  =  1.  
    p_of_megan2cbm05(146) = is_met_iodide           ; p_of_cbm05(146) = non_react  ; cbm05_per_megan(146)  =  1.  
    p_of_megan2cbm05(147) = is_hydrogen_s           ; p_of_cbm05(147) = non_react  ; cbm05_per_megan(147)  =  1.  
    p_of_megan2cbm05(148) = is_met_mercaptan        ; p_of_cbm05(148) = non_react  ; cbm05_per_megan(148)  =  1.  
    p_of_megan2cbm05(149) = is_met_propenyl_2s      ; p_of_cbm05(149) = non_react  ; cbm05_per_megan(149)  =  2.
    p_of_megan2cbm05(150) = is_met_propenyl_2s      ; p_of_cbm05(150) = non_react   ; cbm05_per_megan(150)  =  1.  
    p_of_megan2cbm05(151) = is_pppp_2s              ; p_of_cbm05(151) = non_react  ; cbm05_per_megan(151)  =  2.  
    p_of_megan2cbm05(152) = is_pppp_2s              ; p_of_cbm05(152) = non_react  ; cbm05_per_megan(152)  =  2.  
    p_of_megan2cbm05(153) = is_pppp_2s              ; p_of_cbm05(153) = non_react   ; cbm05_per_megan(153)  =  1.  
    p_of_megan2cbm05(154) = is_2met_nonatriene      ; p_of_cbm05(154) = non_react   ; cbm05_per_megan(154)  =  1.  
    p_of_megan2cbm05(155) = is_met_salicylate       ; p_of_cbm05(155) = non_react  ; cbm05_per_megan(155)  =  1.  
    p_of_megan2cbm05(156) = is_indole               ; p_of_cbm05(156) = non_react  ; cbm05_per_megan(156)  =  1.  
    p_of_megan2cbm05(157) = is_jasmone              ; p_of_cbm05(157) = non_react   ; cbm05_per_megan(157)  =  2.  
    p_of_megan2cbm05(158) = is_met_jasmonate        ; p_of_cbm05(158) = non_react   ; cbm05_per_megan(158)  =  3.  
    p_of_megan2cbm05(159) = is_3met_3dctt           ; p_of_cbm05(159) = non_react   ; cbm05_per_megan(159)  =  3.  
    p_of_megan2cbm05(160) = is_hexanal              ; p_of_cbm05(160) = non_react  ; cbm05_per_megan(160)  =  1.  
    p_of_megan2cbm05(161) = is_hexanal              ; p_of_cbm05(161) = non_react  ; cbm05_per_megan(161)  =  4.  
    p_of_megan2cbm05(162) = is_hexanol_1            ; p_of_cbm05(162) = non_react  ; cbm05_per_megan(162)  =  6.  
    p_of_megan2cbm05(163) = is_hexenal_c3           ; p_of_cbm05(163) = non_react  ; cbm05_per_megan(163)  =  1.  
    p_of_megan2cbm05(164) = is_hexenal_c3           ; p_of_cbm05(164) = non_react  ; cbm05_per_megan(164)  =  3.  
    p_of_megan2cbm05(165) = is_hexenal_c3           ; p_of_cbm05(165) = non_react   ; cbm05_per_megan(165)  =  1   
    p_of_megan2cbm05(166) = is_hexenal_t2           ; p_of_cbm05(166) = non_react  ; cbm05_per_megan(166)  =  1.  
    p_of_megan2cbm05(167) = is_hexenal_t2           ; p_of_cbm05(167) = non_react  ; cbm05_per_megan(167)  =  6.  
    p_of_megan2cbm05(168) = is_hexenal_t2           ; p_of_cbm05(168) = non_react   ; cbm05_per_megan(168)  =  1.  
    p_of_megan2cbm05(169) = is_hexenol_c3           ; p_of_cbm05(169) = non_react  ; cbm05_per_megan(169)  =  5.  
    p_of_megan2cbm05(170) = is_hexenol_c3           ; p_of_cbm05(170) = non_react   ; cbm05_per_megan(170)  =  1.  
    p_of_megan2cbm05(171) = is_hexenyl_act_c3       ; p_of_cbm05(171) = non_react  ; cbm05_per_megan(171)  =  1.  
    p_of_megan2cbm05(172) = is_hexenyl_act_c3       ; p_of_cbm05(172) = non_react  ; cbm05_per_megan(172)  =  5.  
    p_of_megan2cbm05(173) = is_hexenyl_act_c3       ; p_of_cbm05(173) = non_react   ; cbm05_per_megan(173)  =  1.  


  END SUBROUTINE get_megan2sulsoa_global_table
  !--------------------------------------------------------------------



END MODULE module_data_mgn2mech
